@@ -8,7 +8,7 @@ m.Template = m.Class({
 	_init: function(template)
 	{
 		this._template = template;
-		this._tpCache = {};
+		this._tpCache = new Map();
 	},
 	
 	// helper function to return a template value, optionally adjusting for tech.
@@ -16,12 +16,12 @@ m.Template = m.Class({
 	get: function(string)
 	{
 		var value = this._template;
-		if (this._auraTemplateModif && this._auraTemplateModif[string])		{
-			return this._auraTemplateModif[string];
-		} else if (this._techModif && this._techModif[string]) {
-			return this._techModif[string];
+		if (this._auraTemplateModif && this._auraTemplateModif.has(string))		{
+			return this._auraTemplateModif.get(string);
+		} else if (this._techModif && this._techModif.has(string)) {
+			return this._techModif.get(string);
 		} else {
-			if (this._tpCache[string] == null)
+			if (!this._tpCache.has(string))
 			{
 				var args = string.split("/");
 				for (var i = 0; i < args.length; ++i)
@@ -32,10 +32,11 @@ m.Template = m.Class({
 						value = undefined;
 						break;
 					}
-				this._tpCache[string] = value;
+				this._tpCache.set(string, value);
 			}
-			return this._tpCache[string];
-		} 
+			return this._tpCache.get(string);
+		}
+ 
 	},
 
 	genericName: function() {
@@ -51,9 +52,10 @@ m.Template = m.Class({
 	},
 
 	classes: function() {
-		if (!this.get("Identity") || !this.get("Identity/Classes") || !this.get("Identity/Classes/_string"))
+		var template = this.get("Identity");
+		if (!template)
 			return undefined;
-		return this.get("Identity/Classes/_string").split(/\s+/);
+		return GetIdentityClasses(template);
 	},
 	
 	requiredTech: function() {
@@ -379,9 +381,9 @@ m.Template = m.Class({
 
 
 	garrisonableClasses: function() {
-		if (!this.get("GarrisonHolder") || !this.get("GarrisonHolder/List/_string"))
+		if (!this.get("GarrisonHolder"))
 			return undefined;
-		return this.get("GarrisonHolder/List/_string").split(/\s+/);
+		return this.get("GarrisonHolder/List/_string");
 	},
 
 	garrisonMax: function() {
@@ -405,7 +407,13 @@ m.Template = m.Class({
 	buffHeal: function() {
 		if (!this.get("GarrisonHolder"))
 			return undefined;
-		return this.get("GarrisonHolder/BuffHeal");
+		return +this.get("GarrisonHolder/BuffHeal");
+	},
+
+	promotion: function() {
+		if (!this.get("Promotion"))
+			return undefined;
+		return this.get("Promotion/Entity");
 	},
 	
 	/**
@@ -421,15 +429,15 @@ m.Template = m.Class({
 	},
 
 	isHuntable: function() {     // used by Petra
-		if (!this.get("UnitAI") || !this.get("UnitAI/NaturalBehaviour"))
+		if(!this.get("ResourceSupply") || !this.get("ResourceSupply/KillBeforeGather"))
 			return false;
 
 		// special case: rabbits too difficult to hunt for such a small food amount
 		if (this.get("Identity") && this.get("Identity/SpecificName") && this.get("Identity/SpecificName") === "Rabbit")
 			return false;
 
-		// do not hunt retaliating animals.
-		return !(this.get("UnitAI/NaturalBehaviour") === "violent" ||
+		// do not hunt retaliating animals (animals without UnitAI are dead animals)
+		return !this.get("UnitAI") || !(this.get("UnitAI/NaturalBehaviour") === "violent" ||
 			this.get("UnitAI/NaturalBehaviour") === "aggressive" ||
 			this.get("UnitAI/NaturalBehaviour") === "defensive");
 	},
@@ -510,10 +518,10 @@ m.Entity = m.Class({
 
 		this._templateName = entity.template;
 		this._entity = entity;
-		this._auraTemplateModif = {};	// template modification from auras. this is only for this entity.
+		this._auraTemplateModif = new Map();	// template modification from auras. this is only for this entity.
 		this._ai = sharedAI;
 		if (!sharedAI._techModifications[entity.owner][this._templateName])
-			sharedAI._techModifications[entity.owner][this._templateName] = {};
+			sharedAI._techModifications[entity.owner][this._templateName] = new Map();
 		this._techModif = sharedAI._techModifications[entity.owner][this._templateName]; // save a reference to the template tech modifications
 	},
 
@@ -566,11 +574,12 @@ m.Entity = m.Class({
 	
 	// TODO  understand why we have sometimes rounding problems with maxHitpoints ? making wrongly isHurt=true
 	// problem seems to be with hele civs (i.e. spart)
-	hitpoints: function() {if (this._entity.hitpoints !== undefined) return this._entity.hitpoints; return undefined; },
+	hitpoints: function() { if (this._entity.hitpoints !== undefined) return this._entity.hitpoints; return undefined; },
 	isHurt: function() { return (this.hitpoints() + 1) < this.maxHitpoints(); },
 	healthLevel: function() { return (this.hitpoints() / this.maxHitpoints()); },
 	needsHeal: function() { return this.isHurt() && this.isHealable(); },
 	needsRepair: function() { return this.isHurt() && this.isRepairable(); },
+	decaying: function() { if (this._entity.decaying !== undefined) return this._entity.decaying; return undefined; },
 
 	/**
 	 * Returns the current training queue state, of the form
@@ -585,8 +594,10 @@ m.Entity = m.Class({
 		var queue = this._entity.trainingQueue;
 		if (!queue)
 			return undefined;
-		// TODO: compute total time for units in training queue
-		return queue.length;
+		var time = 0;
+		for (var item of queue)
+			time += item.timeRemaining;
+		return time/1000;
 	},
 
 	foundationProgress: function() {
@@ -703,10 +714,16 @@ m.Entity = m.Class({
 		Engine.PostCommand(PlayerID,{"type": "walk", "entities": [this.id()], "x": x, "z": z, "queued": queued });
 		return this;
 	},
-	
-	attackMove: function(x, z, queued) {
+
+	moveToRange: function(x, z, min, max, queued) {
 		queued = queued || false;
-		Engine.PostCommand(PlayerID,{"type": "attack-walk", "entities": [this.id()], "x": x, "z": z, "queued": queued });
+		Engine.PostCommand(PlayerID,{"type": "walk-to-range", "entities": [this.id()], "x": x, "z": z, "min": min, "max": max, "queued": queued });
+		return this;
+	},
+	
+	attackMove: function(x, z, targetClasses, queued) {
+		queued = queued || false;
+		Engine.PostCommand(PlayerID,{"type": "attack-walk", "entities": [this.id()], "x": x, "z": z, "targetClasses": targetClasses, "queued": queued });
 		return this;
 	},
 
@@ -735,8 +752,9 @@ m.Entity = m.Class({
 		return this;
 	},
 
-	garrison: function(target) {
-		Engine.PostCommand(PlayerID,{"type": "garrison", "entities": [this.id()], "target": target.id(),"queued": false});
+	garrison: function(target, queued) {
+		queued = queued || false;
+		Engine.PostCommand(PlayerID,{"type": "garrison", "entities": [this.id()], "target": target.id(),"queued": queued});
 		return this;
 	},
 
@@ -745,6 +763,23 @@ m.Entity = m.Class({
 		return this;
 	},
 	
+	// moveApart from a point in the opposite direction with a distance dist
+	moveApart: function(point, dist) {
+		if (this.position() !== undefined) {
+			var direction = [this.position()[0] - point[0], this.position()[1] - point[1]];
+			var norm = m.VectorDistance(point, this.position());
+			if (norm === 0)
+				direction = [1, 0];
+			else
+			{
+				direction[0] /= norm;
+				direction[1] /= norm;
+			}			
+			Engine.PostCommand(PlayerID,{"type": "walk", "entities": [this.id()], "x": this.position()[0] + direction[0]*dist, "z": this.position()[1] + direction[1]*dist, "queued": false});
+		}
+		return this;
+	},
+
 	// Flees from a unit in the opposite direction.
 	flee: function(unitToFleeFrom) {
 		if (this.position() !== undefined && unitToFleeFrom.position() !== undefined) {
@@ -791,7 +826,18 @@ m.Entity = m.Class({
 		return this;
 	},
 	
-	train: function(type, count, metadata)
+	setRallyPoint: function(target, command) {
+		var data = {"command": command, "target": target.id()};
+		Engine.PostCommand(PlayerID, {"type": "set-rallypoint", "entities": [this.id()], "x": target.position()[0], "z": target.position()[1], "data": data});
+		return this;
+	},
+
+	unsetRallyPoint: function() {
+		Engine.PostCommand(PlayerID, {"type": "unset-rallypoint", "entities": [this.id()]});
+		return this;
+	},
+
+	train: function(type, count, metadata, promotedTypes)
 	{
 		var trainable = this.trainableEntities();
 		if (!trainable)
@@ -810,7 +856,8 @@ m.Entity = m.Class({
 			"entities": [this.id()],
 			"template": type,
 			"count": count,
-			"metadata": metadata
+			"metadata": metadata,
+			"promoted": promotedTypes
 		});
 		return this;
 	},
