@@ -27,6 +27,7 @@ m.GameState.prototype.init = function(SharedScript, state, player) {
 	this.playerData = this.sharedScript.playersData[this.player];
 	this.techModifications = SharedScript._techModifications[this.player];
 	this.barterPrices = SharedScript.barterPrices;
+	this.gameType = SharedScript.gameType;
 };
 
 m.GameState.prototype.update = function(SharedScript, state) {
@@ -119,6 +120,11 @@ m.GameState.prototype.getBarterPrices = function()
 	return this.barterPrices;
 };
 
+m.GameState.prototype.getGameType = function()
+{
+	return this.gameType;
+};
+
 m.GameState.prototype.getTemplate = function(type)
 {
 	if (this.techTemplates[type] !== undefined)
@@ -160,6 +166,8 @@ m.GameState.prototype.cityPhase = function()
 {
 	if (this.playerData.civ == "athen")
 		return "phase_city_athen";
+	else if (this.playerData.civ == "celt")
+		return "phase_city_gauls";
 	return "phase_city_generic";
 };
 
@@ -316,13 +324,19 @@ m.GameState.prototype.getEnemies = function(){
 	return ret;
 };
 
-m.GameState.prototype.getAllies = function(){  // Player is not included
+m.GameState.prototype.getAllies = function(){
 	var ret = [];
-	for (var i in this.playerData.isAlly){
-		if (this.playerData.isAlly[i] && +i !== this.player){
+	for (var i in this.playerData.isAlly)
+		if (this.playerData.isAlly[i])
+			ret.push(i);
+	return ret;
+};
+
+m.GameState.prototype.getExclusiveAllies = function(){  // Player is not included
+	var ret = [];
+	for (var i in this.playerData.isAlly)
+		if (this.playerData.isAlly[i] && +i !== this.player)
 			ret.push(i);
-		}
-	}
 	return ret;
 };
 
@@ -380,6 +394,10 @@ m.GameState.prototype.getAllyEntities = function() {
 	return this.entities.filter(m.Filters.byOwners(this.getAllies()));
 };
 
+m.GameState.prototype.getExclusiveAllyEntities = function() {
+	return this.entities.filter(m.Filters.byOwners(this.getExclusiveAllies()));
+};
+
 // Try to use a parameter for those three, it'll be a lot faster.
 
 m.GameState.prototype.getEnemyEntities = function(enemyID) {
@@ -436,22 +454,25 @@ m.GameState.prototype.countEntitiesByType = function(type, maintain) {
 };
 
 m.GameState.prototype.countEntitiesAndQueuedByType = function(type, maintain) {
+	var template = this.getTemplate(type);
+	if (!template)
+		return 0;
+
 	var count = this.countEntitiesByType(type, maintain);
 	
 	// Count building foundations
-	if (this.getTemplate(type).hasClass("Structure") === true)
+	if (template.hasClass("Structure") === true)
 		count += this.countFoundationsByType(type, true);
-	else if (this.getTemplate(type).resourceSupplyType() !== undefined)	// animal resources
+	else if (template.resourceSupplyType() !== undefined)	// animal resources
 		count += this.countEntitiesByType("resource|" + type, true);
 	else
 	{
 		// Count entities in building production queues
 		// TODO: maybe this fails for corrals.
-		this.getOwnTrainingFacilities().forEach(function(ent){
+		this.getOwnTrainingFacilities().forEach(function(ent) {
 			ent.trainingQueue().forEach(function(item) {
-				if (item.unitTemplate == type){
+				if (item.unitTemplate == type)
 					count += item.count;
-				}
 			});
 		});
 	}
@@ -521,34 +542,64 @@ m.GameState.prototype.getHuntableSupplies = function(){
 	return this.updatingGlobalCollection("resource-hunt", m.Filters.isHuntable(), this.getEntities(), true);
 };
 
+m.GameState.prototype.getFishableSupplies = function(){
+	return this.updatingGlobalCollection("resource-fish", m.Filters.isFishable(), this.getEntities(), true);
+};
+
 // This returns only units from buildings.
-m.GameState.prototype.findTrainableUnits = function(classes){
+m.GameState.prototype.findTrainableUnits = function(classes, anticlasses)
+{
 	var allTrainable = [];
 	this.getOwnStructures().forEach(function(ent) {
 		var trainable = ent.trainableEntities();
-		for (var i in trainable){
-			if (allTrainable.indexOf(trainable[i]) === -1) {
-				allTrainable.push(trainable[i]);
-			}
-		}
+		if (!trainable)
+			return;
+		for (var unit of trainable)
+			if (allTrainable.indexOf(unit) === -1)
+				allTrainable.push(unit);
 	});
 	var ret = [];
-	for (var i in allTrainable) {
+	var limits = this.getEntityLimits();
+	var current = this.getEntityCounts();
+	for (var i in allTrainable)
+	{
 		var template = this.getTemplate(allTrainable[i]);
 
-		if (template.hasClass("Hero"))	// disabling heroes for now
-			continue;
-
 		if (!template.available(this))
 			continue;
 		
 		var okay = true;
-		for (var o in classes)
-			if (!template.hasClass(classes[o]))
-				okay = false;
+		for (var clas of classes)
+		{
+			if (template.hasClass(clas))
+				continue;
+			okay = false;
+			break;
+		}
+		if (!okay)
+			continue;
 
-		if (okay)
-			ret.push( [allTrainable[i], template] );
+		for (var clas of anticlasses)
+		{
+			if (!template.hasClass(clas))
+				continue;
+			okay = false;
+			break;
+		}
+		if (!okay)
+			continue;
+
+		for (var limitedClass in limits)
+		{
+			if (!template.hasClass(limitedClass) || current[limitedClass] < limits[limitedClass])
+				continue;
+			okay = false;
+			break;
+		}
+		if (!okay)
+			continue;
+
+		ret.push( [allTrainable[i], template] );
 	}
 	return ret;
 };
