@@ -51,7 +51,7 @@ m.BaseManager.prototype.init = function(gameState, unconstructed)
 	this.dropsites = {};
 	this.dropsiteSupplies = {};
 	this.gatherers = {};
-	for each (var type in this.Config.resources)
+	for (var type of this.Config.resources)
 	{
 		this.dropsiteSupplies[type] = {"nearby": [], "medium": [], "faraway": []};
 		this.gatherers[type] = {"nextCheck": 0, "used": 0, "lost": 0};
@@ -78,6 +78,13 @@ m.BaseManager.prototype.setAnchor = function(gameState, anchorEntity)
 	this.anchor.setMetadata(PlayerID, "baseAnchor", true);
 	this.buildings.updateEnt(this.anchor);
 	this.accessIndex = gameState.ai.accessibility.getAccessValue(this.anchor.position());
+	// in case all our other bases were destroyed, reaffect these destroyed bases to this base
+	for each (var base in gameState.ai.HQ.baseManagers)
+	{
+		if (base.anchor || base.newbase)
+			continue;
+		base.newbase = this.ID;
+	}
 	return true;
 };
 
@@ -88,9 +95,9 @@ m.BaseManager.prototype.checkEvents = function (gameState, events, queues)
 	var createEvents = events["Create"];
 	var cFinishedEvents = events["ConstructionFinished"];
 
-	for (var i in renameEvents)
+	for (var evt of renameEvents)
 	{
-		var ent = gameState.getEntityById(renameEvents[i].newentity);
+		var ent = gameState.getEntityById(evt.newentity);
 		if (!ent)
 			continue;
 		var workerObject = ent.getMetadata(PlayerID, "worker-object");
@@ -98,11 +105,10 @@ m.BaseManager.prototype.checkEvents = function (gameState, events, queues)
 			workerObject.ent = ent;
 	}
 
-	for (var i in destEvents)
+	for (var evt of destEvents)
 	{
-		var evt = destEvents[i];
 		// let's check we haven't lost an important building here.
-		if (evt != undefined && !evt.SuccessfulFoundation && evt.entityObj != undefined && evt.metadata !== undefined && evt.metadata[PlayerID] &&
+		if (evt && !evt.SuccessfulFoundation && evt.entityObj != undefined && evt.metadata !== undefined && evt.metadata[PlayerID] &&
 			evt.metadata[PlayerID]["base"] !== undefined && evt.metadata[PlayerID]["base"] == this.ID)
 		{
 			var ent = evt.entityObj;
@@ -110,47 +116,62 @@ m.BaseManager.prototype.checkEvents = function (gameState, events, queues)
 				this.removeDropsite(gameState, ent);
 			if (evt.metadata[PlayerID]["baseAnchor"] && evt.metadata[PlayerID]["baseAnchor"] == true)
 			{
-				// sounds like we lost our anchor. Let's try rebuilding it.
-				// TODO: currently the HQ manager sets us as initgathering, we probably ouht to do it
+				// sounds like we lost our anchor. Let's reaffect our units and buildings
 				this.anchor = undefined;
-				
-				this.constructing = true;	// let's switch mode.
-				this.workers.forEach( function (worker) { worker.stopMoving(); });
-				queues.civilCentre.addItem(new m.ConstructionPlan(gameState, gameState.ai.HQ.bBase[0], { "base": this.ID, "baseAnchor": true }, ent.position()));
+				var distmin = Math.min();
+				var basemin = undefined;
+				for each (var base in gameState.ai.HQ.baseManagers)
+				{
+					if (!base.anchor)
+						continue;
+					var dist = API3.SquareVectorDistance(base.anchor.position(), ent.position());
+					if (base.accessIndex !== this.accessIndex)
+						dist += 100000000;
+					if (dist > distmin)
+						continue;
+					distmin = dist;
+					basemin = base;
+				}
+				if (!basemin)
+				{
+					if (this.Config.debug > 1)
+						API3.warn(" base " + this.ID + " destroyed and no other bases found");
+					continue;
+				}
+				this.newbase = basemin.ID;
+				this.units.forEach( function (ent) { ent.setMetadata(PlayerID, "base", basemin.ID); });
+				this.buildings.forEach( function (ent) { ent.setMetadata(PlayerID, "base", basemin.ID); });
 			}
 			
 		}
 	}
-	for (var i in cFinishedEvents)
-	{
-		var evt = cFinishedEvents[i];
-		if (evt && evt.newentity)
-		{
-			var ent = gameState.getEntityById(evt.newentity);
-			if (ent === undefined)
-				continue;
 
-			if (evt.newentity === evt.entity)  // repaired building
-				continue;
+	for (var evt of cFinishedEvents)
+	{
+		if (!evt || !evt.newentity)
+			continue;
+		var ent = gameState.getEntityById(evt.newentity);
+		if (ent === undefined)
+			continue;
+		if (evt.newentity === evt.entity)  // repaired building
+			continue;
 			
-			if (ent.getMetadata(PlayerID,"base") == this.ID)
-			{
-				if (ent.resourceDropsiteTypes() && !ent.hasClass("Elephant"))
-					this.assignResourceToDropsite(gameState, ent);
-			}
+		if (ent.getMetadata(PlayerID,"base") == this.ID)
+		{
+			if (ent.resourceDropsiteTypes() && !ent.hasClass("Elephant"))
+				this.assignResourceToDropsite(gameState, ent);
 		}
 	}
-	for (var i in createEvents)
-	{
-		var evt = createEvents[i];
-		if (evt && evt.entity)
-		{
-			var ent = gameState.getEntityById(evt.entity);
-			if (ent === undefined)
-				continue;
 
-			// do necessary stuff here
-		}
+	for (var evt of createEvents)
+	{
+		if (!evt || !evt.entity)
+			continue;
+		var ent = gameState.getEntityById(evt.entity);
+		if (ent === undefined)
+			continue;
+
+		// do necessary stuff here
 	}
 };
 
@@ -162,14 +183,14 @@ m.BaseManager.prototype.assignResourceToDropsite = function (gameState, dropsite
 {
 	if (this.dropsites[dropsite.id()])
 	{
-		if (this.Config.debug)
+		if (this.Config.debug > 1)
 			warn("assignResourceToDropsite: dropsite already in the list. Should never happen");
 		return;
 	}
 	this.dropsites[dropsite.id()] = true;
 
 	var self = this;
-	for each (var type in dropsite.resourceDropsiteTypes())
+	for (var type of dropsite.resourceDropsiteTypes())
 	{
 		var resources = gameState.getResourceSupplies(type);
 		if (resources.length === 0)
@@ -189,20 +210,21 @@ m.BaseManager.prototype.assignResourceToDropsite = function (gameState, dropsite
 				return;
 			if (supply.hasClass("Field"))     // fields are treated separately
 				return;
-			// quickscope accessibility check
-			if (!gameState.ai.accessibility.pathAvailable(gameState, dropsite.position(), supply.position(),false, true))
+			if (supply.resourceSupplyType()["generic"] === "treasure")  // treasures are treated separately
+				return;
+			// quick accessibility check
+			var access = supply.getMetadata(PlayerID, "access");
+			if (!access)
+			{
+				access = gameState.ai.accessibility.getAccessValue(supply.position());
+				supply.setMetadata(PlayerID, "access", access);
+			}
+			if (access !== self.accessIndex)
 				return;
 
 			var dist = API3.SquareVectorDistance(supply.position(), dropsite.position());
 			if (dist < self.maxDistResourceSquare)
 			{
-				if (supply.resourceSupplyType()["generic"] == "treasure")
-				{
-					if (dist < self.maxDistResourceSquare/4)
-						dist = 0;
-					else
-						dist = self.maxDistResourceSquare/16;
-				}
 				if (dist < self.maxDistResourceSquare/16)        // distmax/4
 				    nearby.push({ "dropsite": dropsite.id(), "id": supply.id(), "ent": supply, "dist": dist }); 
 				else if (dist < self.maxDistResourceSquare/4)    // distmax/2
@@ -266,28 +288,30 @@ m.BaseManager.prototype.removeDropsite = function (gameState, ent)
 m.BaseManager.prototype.findBestDropsiteLocation = function(gameState, resource)
 {
 	
-	var storeHousePlate = gameState.getTemplate(gameState.applyCiv("structures/{civ}_storehouse"));
+	var template = gameState.getTemplate(gameState.applyCiv("structures/{civ}_storehouse"));
 
 	// This builds a map. The procedure is fairly simple. It adds the resource maps
 	//	(which are dynamically updated and are made so that they will facilitate DP placement)
 	// Then checks for a good spot in the territory. If none, and town/city phase, checks outside
 	// The AI will currently not build a CC if it wouldn't connect with an existing CC.
-	
-	var obstructions = m.createObstructionMap(gameState, this.accessIndex, storeHousePlate);
+
+	var obstructions = m.createObstructionMap(gameState, this.accessIndex, template);
 	obstructions.expandInfluences();
 	
-	var locateMap = new API3.Map(gameState.sharedScript);
-
 	var DPFoundations = gameState.getOwnFoundations().filter(API3.Filters.byType(gameState.applyCiv("foundation|structures/{civ}_storehouse")));
 
-	var ccEnts = gameState.getOwnEntities().filter(API3.Filters.byClass("CivCentre")).toEntityArray();
-	
-	// TODO: might be better to check dropsites someplace else.
-	// loop over this in this.terrytoryindices. It's usually a little too much, but it's always enough.
-	var width = locateMap.width;
+	var ccEnts = gameState.getOwnStructures().filter(API3.Filters.byClass("CivCentre")).toEntityArray();
+
+	var width = obstructions.width;
+	var bestIdx = undefined;
+	var bestVal = undefined;
+	var radius = Math.ceil(template.obstructionRadius() / gameState.cellSize);
+
 	for (var p = 0; p < this.territoryIndices.length; ++p)
 	{
 		var j = this.territoryIndices[p];
+		if (obstructions.map[j] <= radius)  // check room around
+			continue;
 		
 		// we add 3 times the needed resource and once the other two (not food)
 		var total = 0;
@@ -340,7 +364,7 @@ m.BaseManager.prototype.findBestDropsiteLocation = function(gameState, resource)
 		if (total == 0)
 			continue;
 
-		for each (var cc in ccEnts)
+		for (var cc of ccEnts)
 		{
 			var ccPos = cc.position();
 			if (!ccPos)
@@ -354,22 +378,23 @@ m.BaseManager.prototype.findBestDropsiteLocation = function(gameState, resource)
 			else if (dist < 6400)
 				total /= 2;
 		}
+		if (total == 0)
+			continue;
 
-		locateMap.map[j] = total;
+		if (bestVal !== undefined && total < bestVal)
+			continue;
+		bestVal = total;
+		bestIdx = j;
 	}
-	
-	var best = locateMap.findBestTile(2, obstructions);
-	var bestIdx = best[0];
 
-	if (this.Config.debug == 2)
-		warn(" for dropsite best is " + best[1]);
+	if (this.Config.debug > 2)
+		warn(" for dropsite best is " + bestVal);
 
-	var quality = best[1];
-	if (quality <= 0)
-		return {"quality": quality, "pos": [0, 0]};
+	if (bestVal <= 0)
+		return {"quality": bestVal, "pos": [0, 0]};
 	var x = ((bestIdx % width) + 0.5) * gameState.cellSize;
 	var z = (Math.floor(bestIdx / width) + 0.5) * gameState.cellSize;
-	return {"quality": quality, "pos": [x, z]};
+	return {"quality": bestVal, "pos": [x, z]};
 };
 
 m.BaseManager.prototype.getResourceLevel = function (gameState, type)
@@ -377,7 +402,7 @@ m.BaseManager.prototype.getResourceLevel = function (gameState, type)
 	var count = 0;
 	var check = {};
 	var nearby = this.dropsiteSupplies[type]["nearby"];
-	for each (var supply in nearby)
+	for (var supply of nearby)
 	{
 		if (check[supply.id])    // avoid double counting as same resource can appear several time
 			continue;
@@ -385,7 +410,7 @@ m.BaseManager.prototype.getResourceLevel = function (gameState, type)
 		count += supply.ent.resourceSupplyAmount();
 	}
 	var medium = this.dropsiteSupplies[type]["medium"];
-	for each (var supply in medium)
+	for (var supply of medium)
 	{
 		if (check[supply.id])
 			continue;
@@ -398,7 +423,7 @@ m.BaseManager.prototype.getResourceLevel = function (gameState, type)
 // check our resource levels and react accordingly
 m.BaseManager.prototype.checkResourceLevels = function (gameState, queues)
 {
-	for each (var type in this.Config.resources)
+	for (var type of this.Config.resources)
 	{
 		if (type == "food")
 		{
@@ -422,9 +447,10 @@ m.BaseManager.prototype.checkResourceLevels = function (gameState, queues)
 				for (var i in queues.field.queue)
 					queues.field.queue[i].isGo = function() { return true; };	// start them
 			}
-			else if(gameState.ai.HQ.canBuild(gameState, "structures/{civ}_field"))	// let's see if we need to add new farms.
+			else if (gameState.ai.HQ.canBuild(gameState, "structures/{civ}_field"))	// let's see if we need to add new farms.
 			{
-				if (numFound < 2 && numFound + numQueue < 3)
+				if ((!gameState.ai.HQ.saveResources && numFound < 2 && numFound + numQueue < 3) ||
+					(gameState.ai.HQ.saveResources && numFound < 1 && numFound + numQueue < 2))
 					queues.field.addItem(new m.ConstructionPlan(gameState, "structures/{civ}_field", { "base" : this.ID }));
 			}
 		}
@@ -448,13 +474,15 @@ m.BaseManager.prototype.checkResourceLevels = function (gameState, queues)
 					{
 						var newDP = this.findBestDropsiteLocation(gameState, type);
 						if (newDP.quality > 50 && gameState.ai.HQ.canBuild(gameState, "structures/{civ}_storehouse"))
-						{
 							queues.dropsites.addItem(new m.ConstructionPlan(gameState, "structures/{civ}_storehouse", { "base": this.ID }, newDP.pos));
-							if (!gameState.isResearched("gather_capacity_wheelbarrow") && !gameState.isResearching("gather_capacity_wheelbarrow"))
-								queues.minorTech.addItem(new m.ResearchPlan(gameState, "gather_capacity_wheelbarrow"));
+						else if (gameState.countFoundationsByType(gameState.ai.HQ.bBase[0], true) === 0 && queues.civilCentre.length() === 0)
+						{
+							// No good dropsite, try to build a new base if no base already planned,
+							// and if not possible, be less strict on dropsite quality
+							if (!gameState.ai.HQ.buildNewBase(gameState, queues, type) && newDP.quality > Math.min(25, 50*0.15/ratio)
+								&& gameState.ai.HQ.canBuild(gameState, "structures/{civ}_storehouse"))
+								queues.dropsites.addItem(new m.ConstructionPlan(gameState, "structures/{civ}_storehouse", { "base": this.ID }, newDP.pos));
 						}
-						else
-							gameState.ai.HQ.buildNewBase(gameState, queues, type);
 					}
 					this.gatherers[type].nextCheck = gameState.ai.playedTurn + 20;
 					this.gatherers[type].used = 0;
@@ -495,6 +523,16 @@ m.BaseManager.prototype.getGatherRates = function(gameState, currentRates)
 		{
 			units = this.workers.filter(API3.Filters.byMetadata(PlayerID, "subrole", "hunter"));
 			units.forEach(function (ent) {
+				if (ent.isIdle())
+					return;
+				var gRate = ent.currentGatherRate()
+				if (gRate !== undefined)
+					currentRates[i] += Math.log(1+gRate)/1.1;
+			});
+			units = this.workers.filter(API3.Filters.byMetadata(PlayerID, "subrole", "fisher"));
+			units.forEach(function (ent) {
+				if (ent.isIdle())
+					return;
 				var gRate = ent.currentGatherRate()
 				if (gRate !== undefined)
 					currentRates[i] += Math.log(1+gRate)/1.1;
@@ -510,7 +548,7 @@ m.BaseManager.prototype.assignRolelessUnits = function(gameState)
 	var roleless = this.units.filter(API3.Filters.not(API3.Filters.byHasMetadata(PlayerID, "role")));
 	var self = this;
 	roleless.forEach(function(ent) {
-		if (ent.hasClass("Worker") || ent.hasClass("CitizenSoldier"))
+		if (ent.hasClass("Worker") || ent.hasClass("CitizenSoldier") || ent.hasClass("FishingBoat"))
 			ent.setMetadata(PlayerID, "role", "worker");
 		else if (ent.hasClass("Support") && ent.hasClass("Elephant"))
 			ent.setMetadata(PlayerID, "role", "worker");
@@ -524,33 +562,45 @@ m.BaseManager.prototype.setWorkersIdleByPriority = function(gameState)
 {
 	if (gameState.currentPhase() < 2)
 		return;
-	
-	var resources = gameState.ai.queueManager.getAvailableResources(gameState);
-
-	var avgOverdraft = 0;
-	for each (var type in resources.types)
-		avgOverdraft += resources[type];
-	avgOverdraft /= 4;
 
+	// change resource only towards one which is more needed, and if changing will not change this order
+	var nb = 1;    // no more than 1 change per turn (otherwise we should update the rates)
 	var mostNeeded = gameState.ai.HQ.pickMostNeededResources(gameState);
-	for each (var type in resources.types)
+	var sumWanted = 0;
+	var sumCurrent = 0;
+	for (var need of mostNeeded)
 	{
-		if (type === mostNeeded[0])
-			continue;
-		if (resources[type] > avgOverdraft + 200 || (resources[type] > avgOverdraft && avgOverdraft > 200))
+		sumWanted += need.wanted;
+		sumCurrent += need.current;
+	}
+	var scale = 1;
+	if (sumWanted > 0)
+		scale = sumCurrent / sumWanted;
+
+	for (var i = mostNeeded.length-1; i > 0; --i)
+	{
+		var lessNeed = mostNeeded[i];
+		for (var j = 0; j < i; ++j) 
 		{
-			if (this.gatherersByType(gameState, type).length === 0)
+			var moreNeed = mostNeeded[j];
+			var lastFailed = gameState.ai.HQ.lastFailedGather[moreNeed.type];
+			if (lastFailed && gameState.ai.elapsedTime - lastFailed < 20)
 				continue;
-			// TODO: perhaps change this?
-			var nb = 2;
-			this.gatherersByType(gameState, type).forEach( function (ent) {
-				if (nb == 0)
+			// If we assume a mean rate of 0.5 per gatherer, this diff should be > 1
+			// but we require a bit more to avoid too frequent changes
+			if ((scale*moreNeed.wanted - moreNeed.current) - (scale*lessNeed.wanted - lessNeed.current) > 1.5)
+			{
+				this.gatherersByType(gameState, lessNeed.type).forEach( function (ent) {
+					if (nb === 0)
+						return;
+					--nb;
+					ent.stopMoving();
+					ent.setMetadata(PlayerID, "gather-type", moreNeed.type);
+					m.AddTCRessGatherer(gameState, moreNeed.type);
+				});
+				if (nb === 0)
 					return;
-				nb--;
-				// TODO: might want to direct assign.
-				ent.stopMoving();
-				ent.setMetadata(PlayerID, "subrole","idle");
-			});
+			}
 		}
 	}
 };
@@ -576,27 +626,30 @@ m.BaseManager.prototype.reassignIdleWorkers = function(gameState)
 				ent.setMetadata(PlayerID, "subrole", "idle");
 				return;
 			}
+
 			if (ent.hasClass("Worker"))
 			{
 				if (self.anchor && self.anchor.needsRepair() === true)
 					ent.repair(self.anchor);
 				else
 				{
-					var types = gameState.ai.HQ.pickMostNeededResources(gameState);
-					for (var i = 0; i < types.length; ++i)
+					var mostNeeded = gameState.ai.HQ.pickMostNeededResources(gameState);
+					for (var needed of mostNeeded)
 					{
-						var lastFailed = gameState.ai.HQ.lastFailedGather[types[i]];
-						if (lastFailed && gameState.ai.playedTurn - lastFailed < 20)
+						var lastFailed = gameState.ai.HQ.lastFailedGather[needed.type];
+						if (lastFailed && gameState.ai.elapsedTime - lastFailed < 20)
 							continue;
 						ent.setMetadata(PlayerID, "subrole", "gatherer");
-						ent.setMetadata(PlayerID, "gather-type", types[i]);
-						m.AddTCRessGatherer(gameState, types[i]);
+						ent.setMetadata(PlayerID, "gather-type", needed.type);
+						m.AddTCRessGatherer(gameState, needed.type);
 						break;
 					}
 				}
 			}
 			else if (ent.hasClass("Cavalry"))
 				ent.setMetadata(PlayerID, "subrole", "hunter");
+			else if (ent.hasClass("FishingBoat"))
+				ent.setMetadata(PlayerID, "subrole", "fisher");
 		});
 	}
 };
@@ -616,10 +669,17 @@ m.BaseManager.prototype.gatherersByType = function(gameState, type)
 // They are idled immediatly and their subrole set to idle.
 m.BaseManager.prototype.pickBuilders = function(gameState, workers, number)
 {
-	var filter = API3.Filters.not(API3.Filters.or(
-		API3.Filters.or(API3.Filters.byMetadata(PlayerID, "plan", -2), API3.Filters.byMetadata(PlayerID, "plan", -3)),
-		API3.Filters.or(API3.Filters.isGarrisoned(), API3.Filters.byClass("Cavalry"))));
-	var availableWorkers = this.workers.filter(filter).toEntityArray();
+	var availableWorkers = this.workers.filter(function (ent) {
+		if (!ent.position())
+			return false;
+		if (ent.getMetadata(PlayerID, "plan") === -2 || ent.getMetadata(PlayerID, "plan") === -3)
+			return false;
+		if (ent.getMetadata(PlayerID, "transport"))
+			return false;
+		if (ent.hasClass("Cavalry") || ent.hasClass("Ship"))
+			return false;
+		return true;
+	}).toEntityArray();
 	availableWorkers.sort(function (a,b) {
 		var vala = 0, valb = 0;
 		if (a.getMetadata(PlayerID, "subrole") == "builder")
@@ -627,16 +687,16 @@ m.BaseManager.prototype.pickBuilders = function(gameState, workers, number)
 		if (b.getMetadata(PlayerID, "subrole") == "builder")
 			valb = 100;
 		if (a.getMetadata(PlayerID, "subrole") == "idle")
-			vala = -20;
+			vala = -50;
 		if (b.getMetadata(PlayerID, "subrole") == "idle")
+			valb = -50;
+		if (a.getMetadata(PlayerID, "plan") === undefined)
+			vala = -20;
+		if (b.getMetadata(PlayerID, "plan") === undefined)
 			valb = -20;
-		if (a.getMetadata(PlayerID, "plan") !== undefined)
-			vala = -100;
-		if (b.getMetadata(PlayerID, "plan") !== undefined)
-			valb = -100;
 		return (vala - valb);
 	});
-	var needed = Math.min(number, availableWorkers.length);
+	var needed = Math.min(number, availableWorkers.length - 3);
 	for (var i = 0; i < needed; ++i)
 	{
 		availableWorkers[i].stopMoving();
@@ -667,7 +727,7 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 	if (!foundations.length && !damagedBuildings.length){
 		return;
 	}
-	var workers = this.workers.filter(API3.Filters.not(API3.Filters.byClass("Cavalry")));
+	var workers = this.workers.filter(API3.Filters.not(API3.Filters.or(API3.Filters.byClass("Cavalry"), API3.Filters.byClass("Ship"))));
 	var builderWorkers = this.workersBySubrole(gameState, "builder");
 	var idleBuilderWorkers = this.workersBySubrole(gameState, "builder").filter(API3.Filters.isIdle());
 
@@ -706,13 +766,15 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 	if (this.constructing == true && maxTotalBuilders < 15)
 		maxTotalBuilders = 15;
 	
-	for (var i in foundations)
+	for (var target of foundations)
 	{
-		var target = foundations[i];
-
 		if (target.hasClass("Field"))
 			continue; // we do not build fields
-		
+
+		if (gameState.ai.HQ.isDangerousLocation(target.position()))
+			if (!target.hasClass("CivCentre") && !target.hasClass("StoneWall"))
+				continue;
+
 		var assigned = gameState.getOwnEntitiesByMetadata("target-foundation", target.id()).length;
 		var targetNB = this.Config.Economy.targetNumBuilders;	// TODO: dynamic that.
 		if (target.hasClass("House") || target.hasClass("Market"))
@@ -721,7 +783,7 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 			targetNB = 4;
 		else if (target.hasClass("Fortress"))
 			targetNB = 7;
-		if (target.getMetadata(PlayerID, "baseAnchor") == true)
+		if (target.getMetadata(PlayerID, "baseAnchor") == true || (target.hasClass("Wonder") && gameState.getGameType() === "wonder"))
 			targetNB = 15;
 
 		if (assigned < targetNB)
@@ -740,7 +802,17 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 				});
 				if (assigned + addedToThis < targetNB)
 				{
-					var nonBuilderWorkers = workers.filter(function(ent) { return (ent.getMetadata(PlayerID, "subrole") !== "builder" && ent.position() !== undefined); }).toEntityArray();
+					var nonBuilderWorkers = workers.filter(function(ent) {
+						if (ent.getMetadata(PlayerID, "subrole") === "builder")
+							return false;
+						if (!ent.position())
+							return false;
+						if (ent.getMetadata(PlayerID, "plan") === -2 || ent.getMetadata(PlayerID, "plan") === -3)
+							return false;
+						if (ent.getMetadata(PlayerID, "transport"))
+							return false;
+						return true;
+					}).toEntityArray();
 					var time = target.buildTime();
 					nonBuilderWorkers.sort(function (workerA,workerB)
 					{
@@ -773,30 +845,33 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 	}
 
 	// don't repair if we're still under attack, unless it's like a vital (civcentre or wall) building that's getting destroyed.
-	for (var i in damagedBuildings)
+	for (var target of damagedBuildings)
 	{
-		var target = damagedBuildings[i];
-		if (gameState.defcon() < 5)
-		{
-			if (target.healthLevel() > 0.5 || !target.hasClass("CivCentre") || !target.hasClass("StoneWall"))
+		if (gameState.ai.HQ.isDangerousLocation(target.position()))
+			if (target.healthLevel() > 0.5 || (!target.hasClass("CivCentre") && !target.hasClass("StoneWall")))
 				continue;
-		}
 		else if (noRepair && !target.hasClass("CivCentre"))
 			continue;
 		
-		if (gameState.ai.HQ.territoryMap.getOwner(target.position()) !== PlayerID ||
-			((gameState.ai.HQ.territoryMap.getOwner([target.position()[0] + 10, target.position()[1]]) !== PlayerID)  &&
-			gameState.ai.HQ.territoryMap.getOwner([target.position()[0] - 10, target.position()[1]]) !== PlayerID))
-			continue;  // TODO find a better way to signal a decaying building
+		if (target.decaying())
+			continue;
 		
 		var assigned = gameState.getOwnEntitiesByMetadata("target-foundation", target.id()).length;
 		if (assigned < targetNB/3)
 		{
 			if (builderWorkers.length + addedWorkers < targetNB*2)
-			{	
-				var nonBuilderWorkers = workers.filter(function(ent) { return (ent.getMetadata(PlayerID, "subrole") !== "builder" && ent.position() !== undefined); });
-				if (gameState.defcon() < 5)
-					nonBuilderWorkers = workers.filter(function(ent) { return (ent.getMetadata(PlayerID, "subrole") !== "builder" && ent.hasClass("Female") && ent.position() !== undefined); });
+			{
+				var nonBuilderWorkers = workers.filter(function(ent) {
+					if (ent.getMetadata(PlayerID, "subrole") === "builder")
+						return false;
+					if (!ent.position())
+						return false;
+					if (ent.getMetadata(PlayerID, "plan") === -2 || ent.getMetadata(PlayerID, "plan") === -3)
+						return false;
+					if (ent.getMetadata(PlayerID, "transport"))
+						return false;
+					return true;
+				});
 				var nearestNonBuilders = nonBuilderWorkers.filterNearest(target.position(), targetNB/3 - assigned);
 				
 				nearestNonBuilders.forEach(function(ent) {
@@ -812,12 +887,27 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 
 m.BaseManager.prototype.update = function(gameState, queues, events)
 {
+	if (!this.anchor)   // this base has been destroyed
+	{
+		// transfer possible remaining units (probably they were in training during previous transfers)
+		if (this.newbase)
+		{
+			var newbase = this.newbase;
+			this.units.forEach( function (ent) { ent.setMetadata(PlayerID, "base", newbase); });
+			this.buildings.forEach( function (ent) { ent.setMetadata(PlayerID, "base", newbase); });
+		}
+		return;
+	}
+
+	if (this.anchor && this.anchor.getMetadata(PlayerID, "access") !== this.accessIndex)
+		API3.warn("Petra baseManager " + this.ID + " problem with accessIndex " + this.accessIndex
+			+ " while metadata access is " + this.anchor.getMetadata(PlayerID, "access"));
+
 	Engine.ProfileStart("Base update - base " + this.ID);
-	var self = this;
-	
+
 	this.checkResourceLevels(gameState, queues);
 	this.assignToFoundations(gameState);
-	
+
 	if (this.constructing && this.anchor)
 	{
 		var owner = gameState.ai.HQ.territoryMap.getOwner(this.anchor.position());
@@ -834,22 +924,22 @@ m.BaseManager.prototype.update = function(gameState, queues, events)
 		}
 	}
 
-
 	if (gameState.ai.playedTurn % 2 === 0)
 		this.setWorkersIdleByPriority(gameState);
 
 	this.assignRolelessUnits(gameState);
-		
+	
 	// should probably be last to avoid reallocations of units that would have done stuffs otherwise.
 	this.reassignIdleWorkers(gameState);
 
 	// TODO: do this incrementally a la defense.js
+	var self = this;
 	this.workers.forEach(function(ent) {
 		if (!ent.getMetadata(PlayerID, "worker-object"))
 			ent.setMetadata(PlayerID, "worker-object", new m.Worker(ent));
 		ent.getMetadata(PlayerID, "worker-object").update(self, gameState);
 	});
-		
+
 	Engine.ProfileStop();
 };
 
