@@ -4,11 +4,14 @@ var PETRA = function(m)
 // this defines the medium difficulty
 m.Config = function() {
 	this.difficulty = 2;	// 0 is sandbox, 1 is easy, 2 is medium, 3 is hard, 4 is very hard.
+	// debug level: 0=none, 1=sanity checks, 2=debug; 3=detailed debug
 	this.debug = 0;
 
+	this.popScaling = 1;  // scale factor depending on the max population
+
 	this.Military = {
 		"towerLapseTime" : 90, // Time to wait between building 2 towers
-		"fortressLapseTime" : 420, // Time to wait between building 2 fortresses
+		"fortressLapseTime" : 390, // Time to wait between building 2 fortresses
 		"popForBarracks1" : 25,
 		"popForBarracks2" : 95,
 		"popForBlacksmith" : 65
@@ -17,13 +20,16 @@ m.Config = function() {
 		"popForTown" : 40,	// How many units we want before aging to town.
 		"cityPhase" : 840,	// time to start trying to reach city phase
 		"popForMarket" : 50,
-		"dockStartTime" : 240,	// Time to wait before building the dock
+		"popForDock" : 25,
 		"targetNumBuilders" : 1.5, // Base number of builders per foundation.
 		"targetNumTraders" : 4, // Target number of traders
+		"targetNumFishers" : 1, // Target number of fishers per sea
 		"femaleRatio" : 0.5, // percent of females among the workforce.
 		"initialFields" : 5
 	};
 
+	this.distUnitGain = 110*110;   // TODO  take it directly from helpers/TraderGain.js
+
 	// Note: attack settings are set directly in attack_plan.js
 	// defense
 	this.Defense =
@@ -31,9 +37,7 @@ m.Config = function() {
 		"defenseRatio" : 2,	// see defense.js for more info.
 		"armyCompactSize" : 2000,	// squared. Half-diameter of an army.
 		"armyBreakawaySize" : 3500,  // squared.
-		"armyMergeSize" : 1400,	// squared.
-		"armyStrengthWariness" : 2,  // Representation of how important army strength is for its "watch level".
-		"prudence" : 1  // Representation of how quickly we'll forget about a dangerous army.
+		"armyMergeSize" : 1400	// squared.
 	};
 	
 	// military
@@ -47,9 +51,12 @@ m.Config = function() {
 		"advanced" : {
 			"default" : [],
 			"athen" : [ "structures/{civ}_gymnasion", "structures/{civ}_prytaneion", "structures/{civ}_theatron" ],
+			"brit" : [ "structures/{civ}_rotarymill" ],
 			"cart" : [ "structures/{civ}_embassy_celtic",
 					"structures/{civ}_embassy_iberian", "structures/{civ}_embassy_italiote" ],
 			"gaul" : [ "structures/{civ}_tavern" ],
+			"hele" : [ "structures/{civ}_gymnasion", "structures/{civ}_prytaneion", "structures/{civ}_theatron" ],
+			"iber" : [ "structures/{civ}_monument" ],
 			"mace" : [ "structures/{civ}_siege_workshop", "structures/{civ}_library", "structures/{civ}_theatron" ],
 			"maur" : [ "structures/{civ}_elephant_stables" ],
 			"pers" : [ "structures/{civ}_stables", "structures/{civ}_apadana" ],
@@ -57,12 +64,17 @@ m.Config = function() {
 			"rome" : [ "structures/{civ}_army_camp" ],
 			"sele" : [ "structures/{civ}_library" ],
 			"spart" : [ "structures/{civ}_syssiton", "structures/{civ}_theatron" ]
+		},
+		"naval" : {
+			"default" : [],
+//			"brit" : [ "structures/{civ}_crannog" ],
+			"cart" : [ "structures/{civ}_super_dock" ]
 		}
 	};
 
 	this.priorities = 
 	{
-		"villager" : 30,	// should be slightly lower than the citizen soldier one because otherwise they get all the food
+		"villager" : 30,      // should be slightly lower than the citizen soldier one to not get all the food
 		"citizenSoldier" : 60,
 		"trader" : 50,
 		"ships" : 70,
@@ -74,13 +86,15 @@ m.Config = function() {
 		"defenseBuilding" : 70,
 		"civilCentre" : 950,
 		"majorTech" : 700,
-		"minorTech" : 40
+		"minorTech" : 40,
+		"emergency" : 1000    // used only in emergency situations, should be the highest one 
 	};
 
 	this.personality =
 	{
 		"aggressive": 0.5,
-		"cooperative": 0.5
+		"cooperative": 0.5,
+		"defensive": 0.5
 	};
 
 	this.resources = ["food", "wood", "stone", "metal"];
@@ -92,7 +106,7 @@ m.Config.prototype.updateDifficulty = function(difficulty)
 {
 	this.difficulty = difficulty;
 	// changing settings based on difficulty.
-	this.targetNumTraders = 2 * this.difficulty;
+	this.Economy.targetNumTraders = 2 * this.difficulty;
 	if (this.difficulty === 1)
 	{
 		this.Military.popForBarracks1 = 35;
