@@ -9,6 +9,7 @@ AIInterface.prototype.EventNames = [
 	"Attacked",
 	"RangeUpdate",
 	"ConstructionFinished",
+	"TrainingStarted",
 	"TrainingFinished",
 	"AIMetadata",
 	"PlayerDefeated",
@@ -128,7 +129,9 @@ AIInterface.prototype.OnGlobalPlayerDefeated = function(msg)
 
 AIInterface.prototype.OnGlobalEntityRenamed = function(msg)
 {
-	this.events["EntityRenamed"].push(msg);
+	var cmpMirage = Engine.QueryInterface(msg.entity, IID_Mirage);
+	if (!cmpMirage)
+		this.events["EntityRenamed"].push(msg);
 };
 
 // When a new technology is researched, check which templates it affects,
@@ -228,4 +231,4 @@ AIInterface.prototype.OnGlobalValueModification = function(msg)
 	}
 };
 
-Engine.RegisterComponentType(IID_AIInterface, "AIInterface", AIInterface);
+Engine.RegisterSystemComponentType(IID_AIInterface, "AIInterface", AIInterface);
