@@ -27,7 +27,7 @@ AuraManager.prototype.ensureExists = function(name, value, id, key, defaultData)
 
 	if (!this[name][value][id][key])
 		this[name][value][id][key] = [];
-}
+};
 
 AuraManager.prototype.ApplyBonus = function(value, ent, data, key)
 {
@@ -127,6 +127,11 @@ AuraManager.prototype.RemoveTemplateBonus = function(value, player, classes, key
 
 AuraManager.prototype.ApplyModifications = function(valueName, value, ent)
 {
+	// FIXME Due do a bug in IonMonkey v24, we must interprete this method
+	// When we upgrade to v31, remove the try-catch line.
+	// See #2684
+	try {} catch(e) {}
+
 	if (!this.modificationsCache[valueName] || !this.modificationsCache[valueName][ent])
 		return value;
 
@@ -137,19 +142,17 @@ AuraManager.prototype.ApplyModifications = function(valueName, value, ent)
 
 AuraManager.prototype.ApplyTemplateModifications = function(valueName, value, player, template)
 {
+	// FIXME Due do a bug in IonMonkey v24, we must interprete this method
+	// When we upgrade to v31, remove the try-catch line.
+	// See #2684
+	try {} catch(e) {}
+
 	if (!this.templateModificationsCache[valueName] || !this.templateModificationsCache[valueName][player])
 		return value;
 
-	var rawClasses;
+	var classes = [];
 	if (template && template.Identity)
-	{
-		rawClasses = template.Identity.Classes;
-		rawClasses = "_string" in rawClasses ?  rawClasses._string : "";
-		if (template.Identity.Rank)
-			rawClasses += " " + template.Identity.Rank;
-	} 
-
-	var classes = rawClasses && rawClasses.length ? rawClasses.split(/\s+/) : [];
+		classes = GetIdentityClasses(template.Identity);
 
 	var keyList = [];
 
@@ -172,4 +175,4 @@ AuraManager.prototype.ApplyTemplateModifications = function(valueName, value, pl
 	return value;
 };
 
-Engine.RegisterComponentType(IID_AuraManager, "AuraManager", AuraManager);
+Engine.RegisterSystemComponentType(IID_AuraManager, "AuraManager", AuraManager);
