@@ -4,6 +4,9 @@ Formation.prototype.Schema =
 	"<element name='FormationName' a:help='Name of the formation'>" +
 		"<text/>" +
 	"</element>" +
+	"<element name='Icon'>" +
+		"<text/>" +
+	"</element>" +
 	"<element name='RequiredMemberCount' a:help='Minimum number of entities the formation should contain'>" +
 		"<data type='nonNegativeInteger'/>" +
 	"</element>" +
@@ -30,6 +33,9 @@ Formation.prototype.Schema =
 	"<element name='WidthDepthRatio' a:help='Average width/depth, counted in number of units.'>" +
 		"<ref name='nonNegativeDecimal'/>" +
 	"</element>" +
+	"<element name='Sloppyness' a:help='Sloppyness in meters (the max difference between the actual and the perfectly aligned formation position'>" +
+		"<ref name='nonNegativeDecimal'/>" +
+	"</element>" +
 	"<optional>" +
 		"<element name='MinColumns' a:help='When possible, this number of colums will be created. Overriding the wanted width depth ratio'>" +
 			"<data type='nonNegativeInteger'/>" +
@@ -77,6 +83,7 @@ Formation.prototype.Init = function()
 		"width": +this.template.UnitSeparationWidthMultiplier,
 		"depth": +this.template.UnitSeparationDepthMultiplier
 	};
+	this.sloppyness = +this.template.Sloppyness;
 	this.widthDepthRatio = +this.template.WidthDepthRatio;
 	this.minColumns = +(this.template.MinColumns || 0);
 	this.maxColumns = +(this.template.MaxColumns || 0);
@@ -253,7 +260,7 @@ Formation.prototype.SetInPosition = function(ent)
 	// Rotate the entity to the right angle
 	var cmpPosition = Engine.QueryInterface(this.entity, IID_Position);
 	var cmpEntPosition = Engine.QueryInterface(ent, IID_Position);
-	if (cmpEntPosition && cmpEntPosition.IsInWorld() && cmpPosition && cmpPosition.IsInWorld)
+	if (cmpEntPosition && cmpEntPosition.IsInWorld() && cmpPosition && cmpPosition.IsInWorld())
 		cmpEntPosition.TurnTo(cmpPosition.GetRotation().y);
 
 	this.inPosition.push(ent);
@@ -289,9 +296,26 @@ Formation.prototype.SetMembers = function(ents)
 
 	var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
 	var templateName = cmpTemplateManager.GetCurrentTemplateName(this.entity);
+	// keep the number of entities per pass class to find the most used
+	// For land units, this will be "default", for ship units, it should be "ship"
+	var passClasses = {};
+	var bestPassClassNumber = 0;
+	var bestPassClass = "default";
 
 	for each (var ent in this.members)
 	{
+		var cmpUnitMotion = Engine.QueryInterface(ent,IID_UnitMotion);
+		var passClass = cmpUnitMotion.GetPassabilityClassName();
+		if (passClasses[passClass])
+			var number = passClasses[passClass]++;
+		else
+			var number = passClasses[passClass] = 1;
+		if (number > bestPassClassNumber)
+		{
+			bestPassClass = passClass;
+			bestPassClassNumber = number;
+		}
+
 		var cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
 		cmpUnitAI.SetFormationController(this.entity);
 		cmpUnitAI.SetLastFormationTemplate(templateName);
@@ -303,6 +327,8 @@ Formation.prototype.SetMembers = function(ents)
 			cmpAuras.ApplyFormationBonus(ents);
 		}
 	}
+	var cmpUnitMotion = Engine.QueryInterface(this.entity, IID_UnitMotion);
+	cmpUnitMotion.SetPassabilityClassName(bestPassClass);
 
 	this.offsets = undefined;
 	// Locate this formation controller in the middle of its members
@@ -499,6 +525,8 @@ Formation.prototype.MoveMembersIntoFormation = function(moveCenter, force)
 
 	var xMax = 0;
 	var yMax = 0;
+	var xMin = 0;
+	var yMin = 0;
 
 	for (var i = 0; i < this.offsets.length; ++i)
 	{
@@ -508,27 +536,20 @@ Formation.prototype.MoveMembersIntoFormation = function(moveCenter, force)
 		if (!cmpUnitAI)
 			continue;
 		
-		if (force)
+		var data = 
 		{
-			cmpUnitAI.ReplaceOrder("FormationWalk", {
-				"target": this.entity,
-				"x": offset.x,
-				"z": offset.y
-			});
-		}
-		else
-		{
-			cmpUnitAI.PushOrderFront("FormationWalk", {
-				"target": this.entity,
-				"x": offset.x,
-				"z": offset.y
-			});
-		}
+			"target": this.entity,
+			"x": offset.x,
+			"z": offset.y
+		};
+		cmpUnitAI.AddOrder("FormationWalk", data, !force);
 		xMax = Math.max(xMax, offset.x);
 		yMax = Math.max(yMax, offset.y);
+		xMin = Math.min(xMin, offset.x);
+		yMin = Math.min(yMin, offset.y);
 	}
-	this.width = xMax * 2;
-	this.depth = yMax * 2;
+	this.width = xMax - xMin;
+	this.depth = yMax - yMin;
 };
 
 Formation.prototype.MoveToMembersCenter = function()
@@ -717,7 +738,14 @@ Formation.prototype.ComputeFormationOffsets = function(active, positions)
 					x += side * centerGap / 2;
 				}
 				var column = Math.ceil(n/2) + Math.ceil(c/2) * side;
-				offsets.push(new Vector2D(x, z));
+				var r1 = 0;
+				var r2 = 0;
+				if (this.sloppyness != 0)
+				{
+					r1 = (Math.random() * 2 - 1) * this.sloppyness;
+					r2 = (Math.random() * 2 - 1) * this.sloppyness;
+				}
+				offsets.push(new Vector2D(x + r1, z + r2));
 				offsets[offsets.length - 1].row = r+1;
 				offsets[offsets.length - 1].column = column;
 				left--
