@@ -3,11 +3,10 @@
 const DEFAULT_NETWORKED_MAP = "Acropolis 01";
 const DEFAULT_OFFLINE_MAP = "Acropolis 01";
 
+const VICTORY_DEFAULTIDX = 1;
+
 // TODO: Move these somewhere like simulation\data\game_types.json, Atlas needs them too
 // Translation: Type of victory condition.
-const VICTORY_TEXT = [translateWithContext("victory", "Conquest"), translateWithContext("victory", "Wonder"), translateWithContext("victory", "None")];
-const VICTORY_DATA = ["conquest", "wonder", "endless"];
-const VICTORY_DEFAULTIDX = 0;
 const POPULATION_CAP = ["50", "100", "150", "200", "250", "300", translate("Unlimited")];
 const POPULATION_CAP_DATA = [50, 100, 150, 200, 250, 300, 10000];
 const POPULATION_CAP_DEFAULTIDX = 5;
@@ -26,7 +25,7 @@ var g_IsNetworked;
 // Is this user in control of game settings (i.e. is a network server, or offline player)
 var g_IsController;
 
-//Server name, if user is a server, connected to the multiplayer lobby
+// Server name, if user is a server, connected to the multiplayer lobby
 var g_ServerName;
 
 // Are we currently updating the GUI in response to network messages instead of user input
@@ -67,11 +66,8 @@ var g_CivData = {};
 
 var g_MapFilters = [];
 
-// Warn about the AI's nonexistent naval map support.
-var g_NavalWarning = "\n\n" + sprintf(
-	translate("%(warning)s The AI does not support naval maps and may cause severe performance issues. Naval maps are recommended to be played with human opponents only."),
-	{ warning: "[font=\"sans-bold-12\"][color=\"orange\"]" + translate("Warning:") + "[/color][/font]" }
-);
+// Current number of assigned human players.
+var g_AssignedCount = 0;
 
 // To prevent the display locking up while we load the map metadata,
 // we'll start with a 'loading' message and switch to the main screen in the
@@ -118,7 +114,7 @@ function init(attribs)
 // Called after the map data is loaded and cached
 function initMain()
 {
-	// Load AI list and hide deprecated AIs
+	// Load AI list
 	g_AIs = Engine.GetAIs();
 
 	// Sort AIs by displayed name
@@ -155,6 +151,8 @@ function initMain()
 	mapFilters.list_data = getFilterIds();
 	g_GameAttributes.mapFilter = "default";
 
+
+
 	// Setup controls for host only
 	if (g_IsController)
 	{
@@ -218,12 +216,16 @@ function initMain()
 		}
 
 		var victoryConditions = Engine.GetGUIObjectByName("victoryCondition");
-		victoryConditions.list = VICTORY_TEXT;
-		victoryConditions.list_data = VICTORY_DATA;
+		var victories = getVictoryConditions();
+		victoryConditions.list = victories.text;
+		victoryConditions.list_data = victories.data;
 		victoryConditions.onSelectionChange = function()
 		{	// Update attributes so other players can see change
 			if (this.selected != -1)
-				g_GameAttributes.settings.GameType = VICTORY_DATA[this.selected];
+			{
+				g_GameAttributes.settings.GameType = victories.data[this.selected];
+				g_GameAttributes.settings.VictoryScripts = victories.scripts[this.selected];
+			}
 
 			if (!g_IsInGuiUpdate)
 				updateGameAttributes();
@@ -281,6 +283,8 @@ function initMain()
 			// Update attributes so other players can see change
 			g_GameAttributes.settings.RatingEnabled = this.checked;
 			Engine.SetRankedGame(this.checked);
+			Engine.GetGUIObjectByName("enableCheats").enabled = !this.checked;
+			Engine.GetGUIObjectByName("lockTeams").enabled = !this.checked;
 			if (!g_IsInGuiUpdate)
 				updateGameAttributes();
 		};
@@ -288,7 +292,6 @@ function initMain()
 	else
 	{
 		// If we're a network client, disable all the map controls
-		// TODO: make them look visually disabled so it's obvious why they don't work
 		Engine.GetGUIObjectByName("mapTypeSelection").hidden = true;
 		Engine.GetGUIObjectByName("mapTypeText").hidden = false;
 		Engine.GetGUIObjectByName("mapFilterSelection").hidden = true;
@@ -304,7 +307,7 @@ function initMain()
 		// TODO: Shouldn't players be able to choose their own assignment?
 		for (var i = 0; i < MAX_PLAYERS; ++i)
 		{
-			Engine.GetGUIObjectByName("playerAssignment["+i+"]").enabled = false;
+			Engine.GetGUIObjectByName("playerAssignment["+i+"]").hidden = true;
 			Engine.GetGUIObjectByName("playerCiv["+i+"]").hidden = true;
 			Engine.GetGUIObjectByName("playerTeam["+i+"]").hidden = true;
 		}
@@ -330,7 +333,10 @@ function initMain()
 		{
 			Engine.GetGUIObjectByName("optionRating").hidden = false;
 			Engine.GetGUIObjectByName("enableRating").checked = Engine.IsRankedGame();
-			g_GameAttributes.settings.RatingEnabled = true;
+			g_GameAttributes.settings.RatingEnabled = Engine.IsRankedGame();
+			// We force locked teams and disabled cheats in ranked games.
+			Engine.GetGUIObjectByName("enableCheats").enabled = !Engine.IsRankedGame();
+			Engine.GetGUIObjectByName("lockTeams").enabled = !Engine.IsRankedGame();
 		}
 		if (g_IsController)
 		{
@@ -395,6 +401,9 @@ function initMain()
 		// to allow easy keyboard selection of maps
 		Engine.GetGUIObjectByName("mapSelection").focus();
 	}
+	// Sync g_GameAttributes to everyone.
+	if (g_IsController)
+		updateGameAttributes();
 }
 
 function handleNetMessage(message)
@@ -423,8 +432,19 @@ function handleNetMessage(message)
 
 	case "gamesetup":
 		if (message.data) // (the host gets undefined data on first connect, so skip that)
+		{
 			g_GameAttributes = message.data;
 
+			// Validate some settings for rated games.
+			if (g_GameAttributes.settings.RatingEnabled)
+			{
+				// Cheats can never be on in rated games.
+				g_GameAttributes.settings.CheatsEnabled = false;
+				// Teams must be locked in rated games.
+				g_GameAttributes.settings.LockTeams = true;
+			}
+		}
+
 		onGameAttributesChange();
 		break;
 
@@ -436,6 +456,9 @@ function handleNetMessage(message)
 		{
 			if (! g_PlayerAssignments[host])
 			{
+				// If we have extra player slots and we are the controller, give the player an ID.
+				if (g_IsController && message.hosts[host].player === -1 && g_AssignedCount < g_GameAttributes.settings.PlayerData.length)
+					Engine.AssignNetworkPlayer(g_AssignedCount + 1, host);
 				addChatMessage({ "type": "connect", "username": message.hosts[host].name });
 				newPlayer = host;
 			}
@@ -845,6 +868,10 @@ function selectMap(name)
 	if (!name)
 		return;
 
+	// reset some map specific properties which are not necessarily redefined on each map
+	g_GameAttributes.settings.TriggerScripts = undefined;
+	g_GameAttributes.settings.CircularMap = undefined;
+
 	var mapData = loadMapData(name);
 	var mapSettings = (mapData && mapData.settings ? deepcopy(mapData.settings) : {});
 
@@ -900,6 +927,10 @@ function launchGame()
 	if (g_GameAttributes.map == "random")
 		selectMap(Engine.GetGUIObjectByName("mapSelection").list_data[Math.floor(Math.random() *
 			(Engine.GetGUIObjectByName("mapSelection").list.length - 1)) + 1]);
+	if (!g_GameAttributes.settings.TriggerScripts)
+		g_GameAttributes.settings.TriggerScripts = g_GameAttributes.settings.VictoryScripts;
+	else
+		g_GameAttributes.settings.TriggerScripts = g_GameAttributes.settings.VictoryScripts.concat(g_GameAttributes.settings.TriggerScripts);
 	g_GameStarted = true;
 	g_GameAttributes.settings.mapType = g_GameAttributes.mapType;
 	var numPlayers = g_GameAttributes.settings.PlayerData.length;
@@ -1002,10 +1033,17 @@ function onGameAttributesChange()
 		var mapSelectionBox = Engine.GetGUIObjectByName("mapSelection");
 		mapSelectionBox.selected = mapSelectionBox.list_data.indexOf(mapName);
 		Engine.GetGUIObjectByName("mapSelectionText").caption = translate(getMapDisplayName(mapName));
-		var populationCapBox = Engine.GetGUIObjectByName("populationCap");
-		populationCapBox.selected = populationCapBox.list_data.indexOf(mapSettings.PopulationCap);
-		var startingResourcesBox = Engine.GetGUIObjectByName("startingResources");
-		startingResourcesBox.selected = startingResourcesBox.list_data.indexOf(mapSettings.StartingResources);
+		if (mapSettings.PopulationCap)
+		{
+			var populationCapBox = Engine.GetGUIObjectByName("populationCap");
+			populationCapBox.selected = populationCapBox.list_data.indexOf(mapSettings.PopulationCap);
+		}		
+		if (mapSettings.StartingResources)
+		{
+			var startingResourcesBox = Engine.GetGUIObjectByName("startingResources");
+			startingResourcesBox.selected = startingResourcesBox.list_data.indexOf(mapSettings.StartingResources);
+		}
+
 		initMapNameList();
 	}
 
@@ -1034,14 +1072,16 @@ function onGameAttributesChange()
 	var startingResourcesText = Engine.GetGUIObjectByName("startingResourcesText");
 	var gameSpeedText = Engine.GetGUIObjectByName("gameSpeedText");
 
+	// We have to check for undefined on these properties as not all maps define them.
 	var sizeIdx = (mapSettings.Size !== undefined && g_MapSizes.tiles.indexOf(mapSettings.Size) != -1 ? g_MapSizes.tiles.indexOf(mapSettings.Size) : g_MapSizes["default"]);
 	var speedIdx = (g_GameAttributes.gameSpeed !== undefined && g_GameSpeeds.speeds.indexOf(g_GameAttributes.gameSpeed) != -1) ? g_GameSpeeds.speeds.indexOf(g_GameAttributes.gameSpeed) : g_GameSpeeds["default"];
-	var victoryIdx = (mapSettings.GameType !== undefined && VICTORY_DATA.indexOf(mapSettings.GameType) != -1 ? VICTORY_DATA.indexOf(mapSettings.GameType) : VICTORY_DEFAULTIDX);
-	enableCheats.checked = (g_GameAttributes.settings.CheatsEnabled === undefined || !g_GameAttributes.settings.CheatsEnabled ? false : true)
+	var victories = getVictoryConditions();
+	var victoryIdx = (mapSettings.GameType !== undefined && victories.data.indexOf(mapSettings.GameType) != -1 ? victories.data.indexOf(mapSettings.GameType) : VICTORY_DEFAULTIDX);
+	enableCheats.checked = (mapSettings.CheatsEnabled === undefined || !mapSettings.CheatsEnabled ? false : true)
 	enableCheatsText.caption = (enableCheats.checked ? "Yes" : "No");
-	if (g_GameAttributes.settings.RatingEnabled !== undefined)
+	if (mapSettings.RatingEnabled !== undefined)
 	{
-		enableRating.checked = g_GameAttributes.settings.RatingEnabled;
+		enableRating.checked = mapSettings.RatingEnabled;
 		Engine.SetRankedGame(enableRating.checked);
 		enableRatingText.caption = (enableRating.checked ? "Yes" : "No");
 	}
@@ -1113,7 +1153,7 @@ function onGameAttributesChange()
 			mapSizeText.caption = g_MapSizes.names[sizeIdx];
 			revealMapText.caption = (mapSettings.RevealMap ? translate("Yes") : translate("No"));
 			exploreMapText.caption = (mapSettings.ExporeMap ? translate("Yes") : translate("No"));
-			victoryConditionText.caption = VICTORY_TEXT[victoryIdx];
+			victoryConditionText.caption = victories.text[victoryIdx];
 			lockTeamsText.caption = (mapSettings.LockTeams ? translate("Yes") : translate("No"));
 		}
 
@@ -1169,7 +1209,7 @@ function onGameAttributesChange()
 
 			revealMapText.caption = (mapSettings.RevealMap ? translate("Yes") : translate("No"));
 			exploreMapText.caption = (mapSettings.ExploreMap ? translate("Yes") : translate("No"));
-			victoryConditionText.caption = VICTORY_TEXT[victoryIdx];
+			victoryConditionText.caption = victories.text[victoryIdx];
 			lockTeamsText.caption = (mapSettings.LockTeams ? translate("Yes") : translate("No"));
 		}
 
@@ -1200,7 +1240,7 @@ function onGameAttributesChange()
 		mapSizeText.caption = translate("Default");
 		revealMapText.caption = (mapSettings.RevealMap ? translate("Yes") : translate("No"));
 		exploreMapText.caption = (mapSettings.ExploreMap ? translate("Yes") : translate("No"));
-		victoryConditionText.caption = VICTORY_TEXT[victoryIdx];
+		victoryConditionText.caption = victories.text[victoryIdx];
 		lockTeamsText.caption = (mapSettings.LockTeams ? translate("Yes") : translate("No"));
 		Engine.GetGUIObjectByName("populationCap").selected = POPULATION_CAP_DEFAULTIDX;
 
@@ -1224,9 +1264,6 @@ function onGameAttributesChange()
 	// Load the description from the map file, if there is one
 	var description = mapSettings.Description ? translate(mapSettings.Description) : translate("Sorry, no description available.");
 
-	if (g_GameAttributes.mapFilter == "naval")
-		description += g_NavalWarning;
-
 	// Describe the number of players
 	var playerString = sprintf(translatePlural("%(number)s player. %(description)s", "%(number)s players. %(description)s", numPlayers), { number: numPlayers, description: description });
 
@@ -1240,6 +1277,8 @@ function onGameAttributesChange()
 			continue;
 
 		var pName = Engine.GetGUIObjectByName("playerName["+i+"]");
+		var pAssignment = Engine.GetGUIObjectByName("playerAssignment["+i+"]");
+		var pAssignmentText = Engine.GetGUIObjectByName("playerAssignmentText["+i+"]");
 		var pCiv = Engine.GetGUIObjectByName("playerCiv["+i+"]");
 		var pCivText = Engine.GetGUIObjectByName("playerCivText["+i+"]");
 		var pTeam = Engine.GetGUIObjectByName("playerTeam["+i+"]");
@@ -1252,13 +1291,22 @@ function onGameAttributesChange()
 
 		// Common to all game types
 		var color = iColorToString(getSetting(pData, pDefs, "Colour"));
-		pColor.sprite = "colour:"+color+" 100";
+		pColor.sprite = "colour:" + color + " 100";
 		pName.caption = translate(getSetting(pData, pDefs, "Name"));
 
 		var team = getSetting(pData, pDefs, "Team");
 		var civ = getSetting(pData, pDefs, "Civ");
 
+		// Nobody but the controller can assign people
+		pAssignmentText.hidden = g_IsController;
+		pAssignment.hidden = !g_IsController;
+		if (!pAssignment.list[0])
+			pAssignmentText.caption = translate("Loading...");
+		else
+			pAssignmentText.caption = pAssignment.list[pAssignment.selected === -1 ? 0 : pAssignment.selected];
+
 		// For clients or scenarios, hide some player dropdowns
+		// TODO: Allow clients to choose their own civ and team
 		if (!g_IsController || g_GameAttributes.mapType == "scenario")
 		{
 			pCivText.hidden = false;
@@ -1298,15 +1346,13 @@ function onGameAttributesChange()
 function updateGameAttributes()
 {
 	if (g_IsNetworked)
-    {
+	{
 		Engine.SetNetworkGameAttributes(g_GameAttributes);
 		if (g_IsController && g_LoadingState >= 2)
 			sendRegisterGameStanza();
 	}
 	else
-    {
 		onGameAttributesChange();
-    }
 }
 
 function AIConfigCallback(ai) 
@@ -1329,7 +1375,7 @@ function updatePlayerList()
 	var assignments = [];
 	var aiAssignments = {};
 	var noAssignment;
-	var assignedCount = 0;
+	g_AssignedCount = 0;
 
 	for (var guid in g_PlayerAssignments)
 	{
@@ -1342,12 +1388,12 @@ function updatePlayerList()
 		assignments[player] = hostID;
 
 		if (player != -1)
-			assignedCount++;
+			g_AssignedCount++;
 	}
 
 	// Only enable start button if we have enough assigned players
 	if (g_IsController)
-		Engine.GetGUIObjectByName("startGame").enabled = (assignedCount > 0);
+		Engine.GetGUIObjectByName("startGame").enabled = (g_AssignedCount > 0);
 
 	for each (var ai in g_AIs)
 	{
@@ -1400,7 +1446,10 @@ function updatePlayerList()
 				if (aiId in aiAssignments)
 					selection = aiAssignments[aiId];
 				else
+				{
+					g_GameAttributes.settings.PlayerData[playerSlot].AI = "";
 					warn(sprintf("AI \"%(id)s\" not present. Defaulting to unassigned.", { id: aiId }));
+				}
 			}
 
 			if (!selection)
@@ -1432,12 +1481,14 @@ function updatePlayerList()
 		}
 
 		var assignBox = Engine.GetGUIObjectByName("playerAssignment["+i+"]");
+		var assignBoxText = Engine.GetGUIObjectByName("playerAssignmentText["+i+"]");
 		assignBox.list = hostNameList;
 		assignBox.list_data = hostGuidList;
 		if (assignBox.selected != selection)
 			assignBox.selected = selection;
+		assignBoxText.caption = hostNameList[selection];
 
-		if (g_IsNetworked && g_IsController)
+		if (g_IsController)
 		{
 			assignBox.onselectionchange = function ()
 			{
@@ -1446,50 +1497,31 @@ function updatePlayerList()
 					var guid = hostGuidList[this.selected];
 					if (guid == "")
 					{
-						// Unassign any host from this player slot
-						Engine.AssignNetworkPlayer(playerID, "");
+						if (g_IsNetworked)
+							// Unassign any host from this player slot
+							Engine.AssignNetworkPlayer(playerID, "");
 						// Remove AI from this player slot
 						g_GameAttributes.settings.PlayerData[playerSlot].AI = "";
 					}
 					else if (guid.substr(0, 3) == "ai:")
 					{
-						// Unassign any host from this player slot
-						Engine.AssignNetworkPlayer(playerID, "");
+						if (g_IsNetworked)
+							// Unassign any host from this player slot
+							Engine.AssignNetworkPlayer(playerID, "");
 						// Set the AI for this player slot
 						g_GameAttributes.settings.PlayerData[playerSlot].AI = guid.substr(3);
 					}
 					else
 						swapPlayers(guid, playerSlot);
 
-					Engine.SetNetworkGameAttributes(g_GameAttributes);
+					if (g_IsNetworked)
+						Engine.SetNetworkGameAttributes(g_GameAttributes);
+					else
+						updatePlayerList();
 					updateReadyUI();
 				}
 			};
 		}
-		else if (!g_IsNetworked)
-		{
-			assignBox.onselectionchange = function ()
-			{
-				if (!g_IsInGuiUpdate)
-				{
-					var guid = hostGuidList[this.selected];
-					if (guid == "")
-					{
-						// Remove AI from this player slot
-						g_GameAttributes.settings.PlayerData[playerSlot].AI = "";
-					}
-					else if (guid.substr(0, 3) == "ai:")
-					{
-						// Set the AI for this player slot
-						g_GameAttributes.settings.PlayerData[playerSlot].AI = guid.substr(3);
-					}
-					else
-						swapPlayers(guid, playerSlot);
-
-					updatePlayerList();
-				}
-			};
-		}
 	}
 
 	g_IsInGuiUpdate = false;
@@ -1636,6 +1668,8 @@ function toggleReady()
 
 function updateReadyUI()
 {
+	if (!g_IsNetworked)
+		return; // Disabled for single-player games.
 	var isAI = new Array(MAX_PLAYERS + 1);
 	for (var i = 0; i < isAI.length; ++i)
 		isAI[i] = true;
@@ -1668,9 +1702,9 @@ function updateReadyUI()
 	}
 
 	// The host is not allowed to start until everyone is ready.
-	var startGameButton = Engine.GetGUIObjectByName("startGame");
 	if (g_IsNetworked && g_IsController)
 	{
+		var startGameButton = Engine.GetGUIObjectByName("startGame");
 		startGameButton.enabled = allReady;
 		// Add a explanation on to the tooltip if disabled.
 		var disabledIndex = startGameButton.tooltip.indexOf('Disabled');
@@ -1819,3 +1853,18 @@ function sendRegisterGameStanza()
 	};
 	Engine.SendRegisterGame(gameData);
 }
+
+function getVictoryConditions()
+{
+	var r = {};
+	r.text = [translate("None")];
+	r.data = ["endless"];
+	r.scripts = [[]];
+	for (var vc in g_VictoryConditions)
+	{
+		r.data.push(vc);
+		r.text.push(g_VictoryConditions[vc].name);
+		r.scripts.push(g_VictoryConditions[vc].scripts);
+	}
+	return r;
+}
