@@ -5,24 +5,23 @@ var g_DebugCommands = false;
 function ProcessCommand(player, cmd)
 {
 	// Do some basic checks here that commanding player is valid
-	var cmpPlayerMan = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
-	if (!cmpPlayerMan || player < 0)
+	var data = {};
+	data.cmpPlayerMan = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
+	if (!data.cmpPlayerMan || player < 0)
 		return;
 
-	var playerEnt = cmpPlayerMan.GetPlayerByID(player);
-	if (playerEnt == INVALID_ENTITY)
+	data.playerEnt = data.cmpPlayerMan.GetPlayerByID(player);
+	if (data.playerEnt == INVALID_ENTITY)
 		return;
 
-	var cmpPlayer = Engine.QueryInterface(playerEnt, IID_Player);
-	if (!cmpPlayer)
+	data.cmpPlayer = Engine.QueryInterface(data.playerEnt, IID_Player);
+	if (!data.cmpPlayer)
 		return;
 
-	var controlAllUnits = cmpPlayer.CanControlAllUnits();
+	data.controlAllUnits = data.cmpPlayer.CanControlAllUnits();
 
-	var entities;
 	if (cmd.entities)
-		entities = FilterEntityList(cmd.entities, player, controlAllUnits);
-
+		data.entities = FilterEntityList(cmd.entities, player, data.controlAllUnits);
 
 	// Note: checks of UnitAI targets are not robust enough here, as ownership
 	//	can change after the order is issued, they should be checked by UnitAI
@@ -31,75 +30,111 @@ function ProcessCommand(player, cmd)
 	//	moves the entities closer to the target before giving up.)
 
 	// Now handle various commands
-	switch (cmd.type)
+	if (commands[cmd.type])
+	{
+		var cmpTrigger = Engine.QueryInterface(SYSTEM_ENTITY, IID_Trigger);
+		cmpTrigger.CallEvent("PlayerCommand", {"player": player, "cmd": cmd});
+		commands[cmd.type](player, cmd, data);
+	}
+	else
+		error("Invalid command: unknown command type: "+uneval(cmd));
+}
+
+var commands = {
+	"debug-print": function(player, cmd, data)
 	{
-	case "debug-print":
 		print(cmd.message);
-		break;
-	
-	case "chat":
-	case "aichat":
+	},
+
+	"chat": function(player, cmd, data)
+	{
 		var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-		cmpGuiInterface.PushNotification({"type": cmd.type, "player": player, "message": cmd.message});
-		break;
-		
-	case "cheat":
+		cmpGuiInterface.PushNotification({"type": cmd.type, "players": [player], "message": cmd.message});
+	},
+
+	"aichat": function(player, cmd, data)
+	{
+		var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
+		cmpGuiInterface.PushNotification({"type": cmd.type, "players": [player], "message": cmd.message});
+	},
+
+	"cheat": function(player, cmd, data)
+	{
 		Cheat(cmd);
-		break;
-		
-	case "quit":
+	},
+
+	"quit": function(player, cmd, data)
+	{
 		// Let the AI exit the game for testing purposes
 		var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-		cmpGuiInterface.PushNotification({"type": "quit"});
-		break;
+		cmpGuiInterface.PushNotification({"type": "quit", "players": [player]});
+	},
 
-	case "diplomacy":
+	"diplomacy": function(player, cmd, data)
+	{
 		switch(cmd.to)
 		{
 		case "ally":
-			cmpPlayer.SetAlly(cmd.player);
+			data.cmpPlayer.SetAlly(cmd.player);
 			break;
 		case "neutral":
-			cmpPlayer.SetNeutral(cmd.player);
+			data.cmpPlayer.SetNeutral(cmd.player);
 			break;
 		case "enemy":
-			cmpPlayer.SetEnemy(cmd.player);
+			data.cmpPlayer.SetEnemy(cmd.player);
 			break;
 		default:
 			warn("Invalid command: Could not set "+player+" diplomacy status of player "+cmd.player+" to "+cmd.to);
 		}
 		var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-		cmpGuiInterface.PushNotification({"type": "diplomacy", "player": player, "player1": cmd.player, "status": cmd.to});
-		break;
+		cmpGuiInterface.PushNotification({"type": "diplomacy", "players": [player], "player1": cmd.player, "status": cmd.to});
+	},
 
-	case "tribute":
-		cmpPlayer.TributeResource(cmd.player, cmd.amounts);
-		break;
+	"tribute": function(player, cmd, data)
+	{
+		data.cmpPlayer.TributeResource(cmd.player, cmd.amounts);
+	},
 
-	case "control-all":
-		cmpPlayer.SetControlAllUnits(cmd.flag);
-		break;
+	"control-all": function(player, cmd, data)
+	{
+		data.cmpPlayer.SetControlAllUnits(cmd.flag);
+	},
 
-	case "reveal-map":
+	"reveal-map": function(player, cmd, data)
+	{
 		// Reveal the map for all players, not just the current player,
 		// primarily to make it obvious to everyone that the player is cheating
 		var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
 		cmpRangeManager.SetLosRevealAll(-1, cmd.enable);
-		break;
+	},
 
-	case "walk":
-		GetFormationUnitAIs(entities, player).forEach(function(cmpUnitAI) {
+	"walk": function(player, cmd, data)
+	{
+		GetFormationUnitAIs(data.entities, player).forEach(function(cmpUnitAI) {
 			cmpUnitAI.Walk(cmd.x, cmd.z, cmd.queued);
 		});
-		break;
+	},
 
-	case "attack-walk":
-		GetFormationUnitAIs(entities, player).forEach(function(cmpUnitAI) {
-			cmpUnitAI.WalkAndFight(cmd.x, cmd.z, cmd.queued);
+	"walk-to-range": function(player, cmd, data)
+	{
+		// Only used by the AI
+		for each (var ent in data.entities)
+		{
+			var cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
+			if(cmpUnitAI)
+				cmpUnitAI.WalkToPointRange(cmd.x, cmd.z, cmd.min, cmd.max, cmd.queued);
+		}
+	},
+
+	"attack-walk": function(player, cmd, data)
+	{
+		GetFormationUnitAIs(data.entities, player).forEach(function(cmpUnitAI) {
+			cmpUnitAI.WalkAndFight(cmd.x, cmd.z, cmd.targetClasses, cmd.queued);
 		});
-		break;
+	},
 
-	case "attack":
+	"attack": function(player, cmd, data)
+	{
 		if (g_DebugCommands && !(IsOwnedByEnemyOfPlayer(player, cmd.target) || IsOwnedByNeutralOfPlayer(player, cmd.target)))
 		{
 			// This check is for debugging only!
@@ -107,12 +142,13 @@ function ProcessCommand(player, cmd)
 		}
 
 		// See UnitAI.CanAttack for target checks
-		GetFormationUnitAIs(entities, player).forEach(function(cmpUnitAI) {
+		GetFormationUnitAIs(data.entities, player).forEach(function(cmpUnitAI) {
 			cmpUnitAI.Attack(cmd.target, cmd.queued);
 		});
-		break;
+	},
 
-	case "heal":
+	"heal": function(player, cmd, data)
+	{
 		if (g_DebugCommands && !(IsOwnedByPlayer(player, cmd.target) || IsOwnedByAllyOfPlayer(player, cmd.target)))
 		{
 			// This check is for debugging only!
@@ -120,12 +156,13 @@ function ProcessCommand(player, cmd)
 		}
 
 		// See UnitAI.CanHeal for target checks
-		GetFormationUnitAIs(entities, player).forEach(function(cmpUnitAI) {
+		GetFormationUnitAIs(data.entities, player).forEach(function(cmpUnitAI) {
 			cmpUnitAI.Heal(cmd.target, cmd.queued);
 		});
-		break;
+	},
 
-	case "repair":
+	"repair": function(player, cmd, data)
+	{
 		// This covers both repairing damaged buildings, and constructing unfinished foundations
 		if (g_DebugCommands && !IsOwnedByAllyOfPlayer(player, cmd.target))
 		{
@@ -134,12 +171,13 @@ function ProcessCommand(player, cmd)
 		}
 
 		// See UnitAI.CanRepair for target checks
-		GetFormationUnitAIs(entities, player).forEach(function(cmpUnitAI) {
+		GetFormationUnitAIs(data.entities, player).forEach(function(cmpUnitAI) {
 			cmpUnitAI.Repair(cmd.target, cmd.autocontinue, cmd.queued);
 		});
-		break;
+	},
 
-	case "gather":
+	"gather": function(player, cmd, data)
+	{
 		if (g_DebugCommands && !(IsOwnedByPlayer(player, cmd.target) || IsOwnedByGaia(cmd.target)))
 		{
 			// This check is for debugging only!
@@ -147,18 +185,20 @@ function ProcessCommand(player, cmd)
 		}
 
 		// See UnitAI.CanGather for target checks
-		GetFormationUnitAIs(entities, player).forEach(function(cmpUnitAI) {
+		GetFormationUnitAIs(data.entities, player).forEach(function(cmpUnitAI) {
 			cmpUnitAI.Gather(cmd.target, cmd.queued);
 		});
-		break;
-		
-	case "gather-near-position":
-		GetFormationUnitAIs(entities, player).forEach(function(cmpUnitAI) {
+	},
+
+	"gather-near-position": function(player, cmd, data)
+	{
+		GetFormationUnitAIs(data.entities, player).forEach(function(cmpUnitAI) {
 			cmpUnitAI.GatherNearPosition(cmd.x, cmd.z, cmd.resourceType, cmd.resourceTemplate, cmd.queued);
 		});
-		break;
+	},
 
-	case "returnresource":
+	"returnresource": function(player, cmd, data)
+	{
 		// Check dropsite is owned by player
 		if (g_DebugCommands && !IsOwnedByPlayer(player, cmd.target))
 		{
@@ -167,30 +207,33 @@ function ProcessCommand(player, cmd)
 		}
 
 		// See UnitAI.CanReturnResource for target checks
-		GetFormationUnitAIs(entities, player).forEach(function(cmpUnitAI) {
+		GetFormationUnitAIs(data.entities, player).forEach(function(cmpUnitAI) {
 			cmpUnitAI.ReturnResource(cmd.target, cmd.queued);
 		});
-		break;
-		
-	case "back-to-work":
-		for each (var ent in entities)
+	},
+
+	"back-to-work": function(player, cmd, data)
+	{
+		for each (var ent in data.entities)
 		{
 			var cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
 			if(!cmpUnitAI || !cmpUnitAI.BackToWork())
 				notifyBackToWorkFailure(player);
 		}
-		break;
+	},
 
-	case "remove-guard":
-		for each (var ent in entities)
+	"remove-guard": function(player, cmd, data)
+	{
+		for each (var ent in data.entities)
 		{
 			var cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
 			if(cmpUnitAI)
 				cmpUnitAI.RemoveGuard();
 		}
-		break;
+	},
 
-	case "train":
+	"train": function(player, cmd, data)
+	{
 		// Check entity limits
 		var cmpTempMan = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
 		var template = cmpTempMan.GetTemplate(cmd.template);
@@ -199,90 +242,111 @@ function ProcessCommand(player, cmd)
 			unitCategory = template.TrainingRestrictions.Category;
 
 		// Verify that the building(s) can be controlled by the player
-		if (entities.length > 0)
+		if (data.entities.length <= 0)
 		{
-			for each (var ent in entities)
+			 if (g_DebugCommands)
+				warn("Invalid command: training building(s) cannot be controlled by player "+player+": "+uneval(cmd));
+			return;
+		}
+
+		for each (var ent in data.entities)
+		{
+			if (unitCategory)
 			{
-				if (unitCategory)
+				var cmpPlayerEntityLimits = QueryOwnerInterface(ent, IID_EntityLimits);
+				if (!cmpPlayerEntityLimits.AllowedToTrain(unitCategory, cmd.count))
 				{
-					var cmpPlayerEntityLimits = QueryOwnerInterface(ent, IID_EntityLimits);
-					if (!cmpPlayerEntityLimits.AllowedToTrain(unitCategory, cmd.count))
+					if (g_DebugCommands)
+						warn(unitCategory + " train limit is reached: " + uneval(cmd));
+					continue;
+				}
+			}
+
+			var cmpTechnologyManager = QueryOwnerInterface(ent, IID_TechnologyManager);
+			if (!cmpTechnologyManager.CanProduce(cmd.template))
+			{
+				if (g_DebugCommands)
+					warn("Invalid command: training requires unresearched technology: " + uneval(cmd));
+				continue;
+			}
+
+			var queue = Engine.QueryInterface(ent, IID_ProductionQueue);
+			// Check if the building can train the unit
+			// TODO: the AI API does not take promotion technologies into account for the list
+			// of trainable units (taken directly from the unit template). Here is a temporary fix. 
+			if (queue && data.cmpPlayer.IsAI())
+			{
+				var list = queue.GetEntitiesList();
+				if (list.indexOf(cmd.template) === -1 && cmd.promoted)
+				{
+					for (var promoted of cmd.promoted)
 					{
-						if (g_DebugCommands)
-							warn(unitCategory + " train limit is reached: " + uneval(cmd));
-						continue;
+						if (list.indexOf(promoted) === -1)
+							continue;
+						cmd.template = promoted;
+						break;
 					}
 				}
-
-				var cmpTechnologyManager = QueryOwnerInterface(ent, IID_TechnologyManager);
-				if (cmpTechnologyManager.CanProduce(cmd.template))
-				{
-					var queue = Engine.QueryInterface(ent, IID_ProductionQueue);
-					// Check if the building can train the unit
-					if (queue && queue.GetEntitiesList().indexOf(cmd.template) != -1)
-						if ("metadata" in cmd)
-							queue.AddBatch(cmd.template, "unit", +cmd.count, cmd.metadata);
-						else
-							queue.AddBatch(cmd.template, "unit", +cmd.count);
-				}
-				else if (g_DebugCommands)
-				{
-					warn("Invalid command: training requires unresearched technology: " + uneval(cmd));
-				}
 			}
+			if (queue && queue.GetEntitiesList().indexOf(cmd.template) != -1)
+				if ("metadata" in cmd)
+					queue.AddBatch(cmd.template, "unit", +cmd.count, cmd.metadata);
+				else
+					queue.AddBatch(cmd.template, "unit", +cmd.count);
 		}
-		else if (g_DebugCommands)
-		{
-			warn("Invalid command: training building(s) cannot be controlled by player "+player+": "+uneval(cmd));
-		}
-		break;
+	},
 
-	case "research":
+	"research": function(player, cmd, data)
+	{
 		// Verify that the building can be controlled by the player
-		if (CanControlUnit(cmd.entity, player, controlAllUnits))
+		if (!CanControlUnit(cmd.entity, player, data.controlAllUnits))
 		{
-			var cmpTechnologyManager = QueryOwnerInterface(cmd.entity, IID_TechnologyManager);
-			if (cmpTechnologyManager.CanResearch(cmd.template))
-			{
-				var queue = Engine.QueryInterface(cmd.entity, IID_ProductionQueue);
-				if (queue)
-					queue.AddBatch(cmd.template, "technology");
-			}
-			else if (g_DebugCommands)
-			{
+			if (g_DebugCommands)
+				warn("Invalid command: research building cannot be controlled by player "+player+": "+uneval(cmd));
+			return;
+		}
+
+		var cmpTechnologyManager = QueryOwnerInterface(cmd.entity, IID_TechnologyManager);
+		if (!cmpTechnologyManager.CanResearch(cmd.template))
+		{
+			if (g_DebugCommands)
 				warn("Invalid command: Requirements to research technology are not met: " + uneval(cmd));
-			}
+			return;
 		}
-		else if (g_DebugCommands)
-		{
-			warn("Invalid command: research building cannot be controlled by player "+player+": "+uneval(cmd));
-		}
-		break;
 
-	case "stop-production":
+		var queue = Engine.QueryInterface(cmd.entity, IID_ProductionQueue);
+		if (queue)
+			queue.AddBatch(cmd.template, "technology");
+	},
+
+	"stop-production": function(player, cmd, data)
+	{
 		// Verify that the building can be controlled by the player
-		if (CanControlUnit(cmd.entity, player, controlAllUnits))
+		if (!CanControlUnit(cmd.entity, player, data.controlAllUnits))
 		{
-			var queue = Engine.QueryInterface(cmd.entity, IID_ProductionQueue);
-			if (queue)
-				queue.RemoveBatch(cmd.id);
+			if (g_DebugCommands)
+				warn("Invalid command: production building cannot be controlled by player "+player+": "+uneval(cmd));
+			return;
 		}
-		else if (g_DebugCommands)
-		{
-			warn("Invalid command: production building cannot be controlled by player "+player+": "+uneval(cmd));
-		}
-		break;
 
-	case "construct":
-		TryConstructBuilding(player, cmpPlayer, controlAllUnits, cmd);
-		break;
+		var queue = Engine.QueryInterface(cmd.entity, IID_ProductionQueue);
+		if (queue)
+			queue.RemoveBatch(cmd.id);
+	},
 
-	case "construct-wall":
-		TryConstructWall(player, cmpPlayer, controlAllUnits, cmd);
-		break;
+	"construct": function(player, cmd, data)
+	{
+		TryConstructBuilding(player, data.cmpPlayer, data.controlAllUnits, cmd);
+	},
 
-	case "delete-entities":
-		for each (var ent in entities)
+	"construct-wall": function(player, cmd, data)
+	{
+		TryConstructWall(player, data.cmpPlayer, data.controlAllUnits, cmd);
+	},
+
+	"delete-entities": function(player, cmd, data)
+	{
+		for each (var ent in data.entities)
 		{
 			var cmpHealth = Engine.QueryInterface(ent, IID_Health);
 			if (cmpHealth)
@@ -294,10 +358,11 @@ function ProcessCommand(player, cmd)
 			else
 				Engine.DestroyEntity(ent);
 		}
-		break;
+	},
 
-	case "set-rallypoint":
-		for each (var ent in entities)
+	"set-rallypoint": function(player, cmd, data)
+	{
+		for each (var ent in data.entities)
 		{
 			var cmpRallyPoint = Engine.QueryInterface(ent, IID_RallyPoint);
 			if (cmpRallyPoint)
@@ -309,93 +374,100 @@ function ProcessCommand(player, cmd)
 				cmpRallyPoint.AddData(cmd.data);
 			}
 		}
-		break;
+	},
 
-	case "unset-rallypoint":
-		for each (var ent in entities)
+	"unset-rallypoint": function(player, cmd, data)
+	{
+		for each (var ent in data.entities)
 		{
 			var cmpRallyPoint = Engine.QueryInterface(ent, IID_RallyPoint);
 			if (cmpRallyPoint)
 				cmpRallyPoint.Reset();
 		}
-		break;
+	},
 
-	case "defeat-player":
+	"defeat-player": function(player, cmd, data)
+	{
 		// Send "OnPlayerDefeated" message to player
-		Engine.PostMessage(playerEnt, MT_PlayerDefeated, { "playerId": player } );
-		break;
+		Engine.PostMessage(data.playerEnt, MT_PlayerDefeated, { "playerId": player } );
+	},
 
-	case "garrison":
+	"garrison": function(player, cmd, data)
+	{
 		// Verify that the building can be controlled by the player or is mutualAlly
-		if (CanControlUnitOrIsAlly(cmd.target, player, controlAllUnits))
+		if (!CanControlUnitOrIsAlly(cmd.target, player, data.controlAllUnits))
 		{
-			GetFormationUnitAIs(entities, player).forEach(function(cmpUnitAI) {
-				cmpUnitAI.Garrison(cmd.target, cmd.queued);
-			});
+			if (g_DebugCommands)
+				warn("Invalid command: garrison target cannot be controlled by player "+player+" (or ally): "+uneval(cmd));
+			return;
 		}
-		else if (g_DebugCommands)
-		{
-			warn("Invalid command: garrison target cannot be controlled by player "+player+" (or ally): "+uneval(cmd));
-		}
-		break;
 
-	case "guard":
+		GetFormationUnitAIs(data.entities, player).forEach(function(cmpUnitAI) {
+			cmpUnitAI.Garrison(cmd.target, cmd.queued);
+		});
+	},
+
+	"guard": function(player, cmd, data)
+	{
 		// Verify that the target can be controlled by the player or is mutualAlly
-		if (CanControlUnitOrIsAlly(cmd.target, player, controlAllUnits))
+		if (!CanControlUnitOrIsAlly(cmd.target, player, data.controlAllUnits))
 		{
-			GetFormationUnitAIs(entities, player).forEach(function(cmpUnitAI) {
-				cmpUnitAI.Guard(cmd.target, cmd.queued);
-			});
+			if (g_DebugCommands)
+				warn("Invalid command: guard/escort target cannot be controlled by player "+player+": "+uneval(cmd));
+			return;
 		}
-		else if (g_DebugCommands)
-		{
-			warn("Invalid command: guard/escort target cannot be controlled by player "+player+": "+uneval(cmd));
-		}
-		break;
 
-	case "stop":
-		GetFormationUnitAIs(entities, player).forEach(function(cmpUnitAI) {
+		GetFormationUnitAIs(data.entities, player).forEach(function(cmpUnitAI) {
+			cmpUnitAI.Guard(cmd.target, cmd.queued);
+		});
+	},
+
+	"stop": function(player, cmd, data)
+	{
+		GetFormationUnitAIs(data.entities, player).forEach(function(cmpUnitAI) {
 			cmpUnitAI.Stop(cmd.queued);
 		});
-		break;
+	},
 
-	case "unload":
+	"unload": function(player, cmd, data)
+	{
 		// Verify that the building can be controlled by the player or is mutualAlly
-		if (CanControlUnitOrIsAlly(cmd.garrisonHolder, player, controlAllUnits))
+		if (!CanControlUnitOrIsAlly(cmd.garrisonHolder, player, data.controlAllUnits))
 		{
-			var cmpGarrisonHolder = Engine.QueryInterface(cmd.garrisonHolder, IID_GarrisonHolder);
-			var notUngarrisoned = 0;
-
-			// The owner can ungarrison every garrisoned unit
-			if (IsOwnedByPlayer(player, cmd.garrisonHolder))
-				entities = cmd.entities;
-
-			for each (var ent in entities)
-				if (!cmpGarrisonHolder || !cmpGarrisonHolder.Unload(ent))
-					notUngarrisoned++;
-
-			if (notUngarrisoned != 0)
-				notifyUnloadFailure(player, cmd.garrisonHolder)
+			if (g_DebugCommands)
+				warn("Invalid command: unload target cannot be controlled by player "+player+" (or ally): "+uneval(cmd));
+			return;
 		}
-		else if (g_DebugCommands)
-		{
-			warn("Invalid command: unload target cannot be controlled by player "+player+" (or ally): "+uneval(cmd));
-		}
-		break;
 
-	case "unload-template":
+		var cmpGarrisonHolder = Engine.QueryInterface(cmd.garrisonHolder, IID_GarrisonHolder);
+		var notUngarrisoned = 0;
+
+		// The owner can ungarrison every garrisoned unit
+		if (IsOwnedByPlayer(player, cmd.garrisonHolder))
+			data.entities = cmd.entities;
+
+		for each (var ent in data.entities)
+			if (!cmpGarrisonHolder || !cmpGarrisonHolder.Unload(ent))
+				notUngarrisoned++;
+
+		if (notUngarrisoned != 0)
+			notifyUnloadFailure(player, cmd.garrisonHolder)
+	},
+
+	"unload-template": function(player, cmd, data)
+	{
 		var index = cmd.template.indexOf("&");  // Templates for garrisoned units are extended
 		if (index == -1)
-			break;
+			return;
 
-		var entities = FilterEntityListWithAllies(cmd.garrisonHolders, player, controlAllUnits);
+		var entities = FilterEntityListWithAllies(cmd.garrisonHolders, player, data.controlAllUnits);
 		for each (var garrisonHolder in entities)
 		{
 			var cmpGarrisonHolder = Engine.QueryInterface(garrisonHolder, IID_GarrisonHolder);
 			if (cmpGarrisonHolder)
 			{
 				// Only the owner of the garrisonHolder may unload entities from any owners
-				if (!IsOwnedByPlayer(player, garrisonHolder) && !controlAllUnits
+				if (!IsOwnedByPlayer(player, garrisonHolder) && !data.controlAllUnits
 				    && player != +cmd.template.slice(1,index))
 						continue;
 
@@ -403,56 +475,62 @@ function ProcessCommand(player, cmd)
 					notifyUnloadFailure(player, garrisonHolder);
 			}
 		}
-		break;
+	},
 
-	case "unload-all-own":
-		var entities = FilterEntityList(cmd.garrisonHolders, player, controlAllUnits);
+	"unload-all-own": function(player, cmd, data)
+	{
+		var entities = FilterEntityList(cmd.garrisonHolders, player, data.controlAllUnits);
 		for each (var garrisonHolder in entities)
 		{
 			var cmpGarrisonHolder = Engine.QueryInterface(garrisonHolder, IID_GarrisonHolder);
 			if (!cmpGarrisonHolder || !cmpGarrisonHolder.UnloadAllOwn())
 				notifyUnloadFailure(player, garrisonHolder)
 		}
-		break;
+	},
 
-	case "unload-all":
-		var entities = FilterEntityList(cmd.garrisonHolders, player, controlAllUnits);
+	"unload-all": function(player, cmd, data)
+	{
+		var entities = FilterEntityList(cmd.garrisonHolders, player, data.controlAllUnits);
 		for each (var garrisonHolder in entities)
 		{
 			var cmpGarrisonHolder = Engine.QueryInterface(garrisonHolder, IID_GarrisonHolder);
 			if (!cmpGarrisonHolder || !cmpGarrisonHolder.UnloadAll())
 				notifyUnloadFailure(player, garrisonHolder)
 		}
-		break;
-	
-	case "increase-alert-level":
-		for each (var ent in entities)
+	},
+
+	"increase-alert-level": function(player, cmd, data)
+	{
+		for each (var ent in data.entities)
 		{
 			var cmpAlertRaiser = Engine.QueryInterface(ent, IID_AlertRaiser);
 			if (!cmpAlertRaiser || !cmpAlertRaiser.IncreaseAlertLevel())
 				notifyAlertFailure(player);
 		}
-		break;
-	
-	case "alert-end":
-		for each (var ent in entities)
+	},
+
+	"alert-end": function(player, cmd, data)
+	{
+		for each (var ent in data.entities)
 		{
 			var cmpAlertRaiser = Engine.QueryInterface(ent, IID_AlertRaiser);
 			if (cmpAlertRaiser)
 				cmpAlertRaiser.EndOfAlert();
 		}
-		break;
+	},
 
-	case "formation":
-		GetFormationUnitAIs(entities, player, cmd.name).forEach(function(cmpUnitAI) {
+	"formation": function(player, cmd, data)
+	{
+		GetFormationUnitAIs(data.entities, player, cmd.name).forEach(function(cmpUnitAI) {
 			cmpUnitAI.MoveIntoFormation(cmd);
 		});
-		break;
+	},
 
-	case "promote":
+	"promote": function(player, cmd, data)
+	{
 		// No need to do checks here since this is a cheat anyway
 		var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-		cmpGuiInterface.PushNotification({"type": "chat", "player": player, "message": "(Cheat - promoted units)"});
+		cmpGuiInterface.PushNotification({"type": "chat", "players": [player], "message": "(Cheat - promoted units)"});
 
 		for each (var ent in cmd.entities)
 		{
@@ -460,26 +538,29 @@ function ProcessCommand(player, cmd)
 			if (cmpPromotion)
 				cmpPromotion.IncreaseXp(cmpPromotion.GetRequiredXp() - cmpPromotion.GetCurrentXp());
 		}
-		break;
+	},
 
-	case "stance":
-		for each (var ent in entities)
+	"stance": function(player, cmd, data)
+	{
+		for each (var ent in data.entities)
 		{
 			var cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
 			if (cmpUnitAI)
 				cmpUnitAI.SwitchToStance(cmd.name);
 		}
-		break;
+	},
 
-	case "wall-to-gate":
-		for each (var ent in entities)
+	"wall-to-gate": function(player, cmd, data)
+	{
+		for each (var ent in data.entities)
 		{
-			TryTransformWallToGate(ent, cmpPlayer, cmd.template);
+			TryTransformWallToGate(ent, data.cmpPlayer, cmd.template);
 		}
-		break;
+	},
 
-	case "lock-gate":
-		for each (var ent in entities)
+	"lock-gate": function(player, cmd, data)
+	{
+		for each (var ent in data.entities)
 		{
 			var cmpGate = Engine.QueryInterface(ent, IID_Gate);
 			if (cmpGate)
@@ -490,32 +571,38 @@ function ProcessCommand(player, cmd)
 					cmpGate.UnlockGate();
 			}
 		}
-		break;
+	},
 
-	case "setup-trade-route":
-		GetFormationUnitAIs(entities, player).forEach(function(cmpUnitAI) {
+	"setup-trade-route": function(player, cmd, data)
+	{
+		GetFormationUnitAIs(data.entities, player).forEach(function(cmpUnitAI) {
 			cmpUnitAI.SetupTradeRoute(cmd.target, cmd.source, cmd.route, cmd.queued);
 		});
+	},
 
-	case "select-required-goods":
-		for each (var ent in entities)
+	"select-required-goods": function(player, cmd, data)
+	{
+		for each (var ent in data.entities)
 		{
 			var cmpTrader = Engine.QueryInterface(ent, IID_Trader);
 			if (cmpTrader)
 				cmpTrader.SetRequiredGoods(cmd.requiredGoods);
 		}
-		break;
+	},
 
-	case "set-trading-goods":
-		cmpPlayer.SetTradingGoods(cmd.tradingGoods);
-		break;
+	"set-trading-goods": function(player, cmd, data)
+	{
+		data.cmpPlayer.SetTradingGoods(cmd.tradingGoods);
+	},
 
-	case "barter":
+	"barter": function(player, cmd, data)
+	{
 		var cmpBarter = Engine.QueryInterface(SYSTEM_ENTITY, IID_Barter);
-		cmpBarter.ExchangeResources(playerEnt, cmd.sell, cmd.buy, cmd.amount);
-		break;
-		
-	case "set-shading-color":
+		cmpBarter.ExchangeResources(data.playerEnt, cmd.sell, cmd.buy, cmd.amount);
+	},
+
+	"set-shading-color": function(player, cmd, data)
+	{
 		// Debug command to make an entity brightly colored
 		for each (var ent in cmd.entities)
 		{
@@ -523,10 +610,11 @@ function ProcessCommand(player, cmd)
 			if (cmpVisual)
 				cmpVisual.SetShadingColour(cmd.rgb[0], cmd.rgb[1], cmd.rgb[2], 0) // alpha isn't used so just send 0
 		}
-		break;
+	},
 
-	case "pack":
-		for each (var ent in entities)
+	"pack": function(player, cmd, data)
+	{
+		for each (var ent in data.entities)
 		{
 			var cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
 			if (cmpUnitAI)
@@ -537,10 +625,11 @@ function ProcessCommand(player, cmd)
 					cmpUnitAI.Unpack(cmd.queued);
 			}
 		}
-		break;
+	},
 
-	case "cancel-pack":
-		for each (var ent in entities)
+	"cancel-pack": function(player, cmd, data)
+	{
+		for each (var ent in data.entities)
 		{
 			var cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
 			if (cmpUnitAI)
@@ -551,12 +640,13 @@ function ProcessCommand(player, cmd)
 					cmpUnitAI.CancelUnpack(cmd.queued);
 			}
 		}
-		break;
-
-	default:
-		error("Invalid command: unknown command type: "+uneval(cmd));
-	}
-}
+	},
+	"dialog-answer": function(player, cmd, data)
+	{
+		// Currently nothing. Triggers can read it anyway, and send this
+		// message to any component you like.
+	},
+};
 
 /**
  * Sends a GUI notification about unit(s) that failed to ungarrison.
@@ -564,7 +654,7 @@ function ProcessCommand(player, cmd)
 function notifyUnloadFailure(player, garrisonHolder)
 {
 	var cmpPlayer = QueryPlayerIDInterface(player, IID_Player);
-	var notification = {"player": cmpPlayer.GetPlayerID(), "message": "Unable to ungarrison unit(s)" };
+	var notification = {"players": [cmpPlayer.GetPlayerID()], "message": "Unable to ungarrison unit(s)" };
 	var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 	cmpGUIInterface.PushNotification(notification);
 }
@@ -575,7 +665,7 @@ function notifyUnloadFailure(player, garrisonHolder)
 function notifyBackToWorkFailure(player)
 {
 	var cmpPlayer = QueryPlayerIDInterface(player, IID_Player);
-	var notification = {"player": cmpPlayer.GetPlayerID(), "message": "Some unit(s) can't go back to work" };
+	var notification = {"players": [cmpPlayer.GetPlayerID()], "message": "Some unit(s) can't go back to work" };
 	var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 	cmpGUIInterface.PushNotification(notification);
 }
@@ -586,7 +676,7 @@ function notifyBackToWorkFailure(player)
 function notifyAlertFailure(player)
 {
 	var cmpPlayer = QueryPlayerIDInterface(player, IID_Player);
-	var notification = {"player": cmpPlayer.GetPlayerID(), "message": "You can't raise the alert to a higher level !" };
+	var notification = {"players": [cmpPlayer.GetPlayerID()], "message": "You can't raise the alert to a higher level !" };
 	var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 	cmpGUIInterface.PushNotification(notification);
 }
@@ -621,31 +711,19 @@ function ExtractFormations(ents)
  * Tries to find the best angle to put a dock at a given position
  * Taken from GuiInterface.js
  */
-function GetDockAngle(templateName,x,y)
+function GetDockAngle(template, x, z)
 {
-	var cmpTemplateMgr = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
-	var template = cmpTemplateMgr.GetTemplate(templateName);
-
-	if (template.BuildRestrictions.Category !== "Dock")
-		return undefined;
-
 	var cmpTerrain = Engine.QueryInterface(SYSTEM_ENTITY, IID_Terrain);
 	var cmpWaterManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_WaterManager);
 	if (!cmpTerrain || !cmpWaterManager)
-	{
 		return undefined;
-	}
 	
 	// Get footprint size
 	var halfSize = 0;
 	if (template.Footprint.Square)
-	{
 		halfSize = Math.max(template.Footprint.Square["@depth"], template.Footprint.Square["@width"])/2;
-	}
 	else if (template.Footprint.Circle)
-	{
 		halfSize = template.Footprint.Circle["@radius"];
-	}
 	
 	/* Find direction of most open water, algorithm:
 	 *	1. Pick points in a circle around dock
@@ -665,28 +743,24 @@ function GetDockAngle(templateName,x,y)
 			var angle = (i/numPoints)*2*Math.PI;
 			var d = halfSize*(dist+1);
 			var nx = x - d*Math.sin(angle);
-			var nz = y + d*Math.cos(angle);
+			var nz = z + d*Math.cos(angle);
 			
 			if (cmpTerrain.GetGroundLevel(nx, nz) < cmpWaterManager.GetWaterLevel(nx, nz))
-			{
 				waterPoints.push(i);
-			}
 		}
 		var consec = [];
 		var length = waterPoints.length;
+		if (!length)
+			continue;
 		for (var i = 0; i < length; ++i)
 		{
 			var count = 0;
 			for (var j = 0; j < (length-1); ++j)
 			{
 				if (((waterPoints[(i + j) % length]+1) % numPoints) == waterPoints[(i + j + 1) % length])
-				{
 					++count;
-				}
 				else
-				{
 					break;
-				}
 			}
 			consec[i] = count;
 		}
@@ -703,9 +777,7 @@ function GetDockAngle(templateName,x,y)
 		
 		// If we've found a shoreline, stop searching
 		if (count != numPoints-1)
-		{
 			return -(((waterPoints[start] + consec[start]/2) % numPoints)/numPoints*2*Math.PI);
-		}
 	}
 	return undefined;
 }
@@ -761,9 +833,14 @@ function TryConstructBuilding(player, cmpPlayer, controlAllUnits, cmd)
 	}
 	
 	// If it's a dock, get the right angle.
-	var angle = GetDockAngle(cmd.template,cmd.x,cmd.z);
-	if (angle !== undefined)
-		cmd.angle = angle;
+	var cmpTemplateMgr = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
+	var template = cmpTemplateMgr.GetTemplate(cmd.template);
+	if (template.BuildRestrictions.Category === "Dock")
+	{
+		var angle = GetDockAngle(template, cmd.x, cmd.z);
+		if (angle !== undefined)
+			cmd.angle = angle;
+	}
 	
 	// Move the foundation to the right place
 	var cmpPosition = Engine.QueryInterface(ent, IID_Position);
@@ -805,7 +882,7 @@ function TryConstructBuilding(player, cmpPlayer, controlAllUnits, cmd)
 			}
 
 			var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-			ret.player = player;
+			ret.players = [player];
 			cmpGuiInterface.PushNotification(ret);
 			
 			// Remove the foundation because the construction was aborted
@@ -844,7 +921,7 @@ function TryConstructBuilding(player, cmpPlayer, controlAllUnits, cmd)
 		}
 		
 		var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-		cmpGuiInterface.PushNotification({ "player": player, "message": "Building's technology requirements are not met." }); 
+		cmpGuiInterface.PushNotification({ "players": [player], "message": "Building's technology requirements are not met." }); 
 		
 		// Remove the foundation because the construction was aborted 
 		cmpPosition.MoveOutOfWorld();
@@ -896,6 +973,11 @@ function TryConstructBuilding(player, cmpPlayer, controlAllUnits, cmd)
 			"queued": cmd.queued
 		});
 	}
+	
+	// Load a mirage for the owner of this new entity
+	var cmpFogging = Engine.QueryInterface(ent, IID_Fogging)
+	if (cmpFogging)
+		cmpFogging.LoadMirage(player);
 
 	return ent;
 }
@@ -1181,9 +1263,10 @@ function GetFormationUnitAIs(ents, player, formationTemplate)
 	var nonformedUnitAIs = [];
 	for each (var ent in ents)
 	{
-		// Skip units with no UnitAI
+		// Skip units with no UnitAI or no position
 		var cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
-		if (!cmpUnitAI)
+		var cmpPosition = Engine.QueryInterface(ent, IID_Position);
+		if (!cmpUnitAI || !cmpPosition || !cmpPosition.IsInWorld())
 			continue;
 
 		var cmpIdentity = Engine.QueryInterface(ent, IID_Identity);
@@ -1305,11 +1388,6 @@ function ClusterEntities(ents, separationDistance)
 		matrix[i] = [];
 		clusters.push([ents[i]]);
 		var cmpPosition = Engine.QueryInterface(ents[i], IID_Position);
-		if (!cmpPosition)
-		{
-			error("Asked to cluster entities without position: "+ents[i]);
-			return clusters;
-		}
 		positions.push(cmpPosition.GetPosition2D());
 		for (var j = 0; j < i; j++)
 			matrix[i][j] = positions[i].distanceToSquared(positions[j]);
@@ -1510,4 +1588,7 @@ function ReplaceBuildingWith(ent, building)
 
 Engine.RegisterGlobal("GetFormationRequirements", GetFormationRequirements);
 Engine.RegisterGlobal("CanMoveEntsIntoFormation", CanMoveEntsIntoFormation);
+Engine.RegisterGlobal("GetDockAngle", GetDockAngle);
 Engine.RegisterGlobal("ProcessCommand", ProcessCommand);
+Engine.RegisterGlobal("commands", commands);
+
