@@ -394,9 +394,9 @@ setSunColour(0.917, 0.828, 0.734);
 setWaterColour(0.292, 0.347, 0.691);		
 setWaterColour(0, 0.501961, 1);		
 setWaterTint(0.501961, 1, 1);
-setWaterWaviness(10);
+setWaterWaviness(2.5);
+setWaterType("ocean");
 setWaterMurkiness(0.49);
-setWaterReflectionTintStrength(0.0);
 
 setFogFactor(0.3);
 setFogThickness(0.25);
