@@ -496,10 +496,9 @@ createStragglerTrees(types, avoidClasses(clWater, 1, clForest, 1, clHill, 1, clP
 setSkySet("cirrus");
 setWaterColour(0.114, 0.192, 0.463);
 setWaterTint(0.255, 0.361, 0.651);
-setWaterReflectionTint(0.255, 0.361, 0.651);
-setWaterWaviness(10);
+setWaterWaviness(3.0);
+setWaterType("ocean");
 setWaterMurkiness(0.83);
-setWaterReflectionTintStrength(0.05);
 
 setFogThickness(0.35);
 setFogFactor(0.55);
