@@ -635,10 +635,9 @@ createObjectGroups(group, 0,
 setSkySet("sunny");
 setWaterColour(0.024,0.212,0.024);
 setWaterTint(0.133, 0.725,0.855);
-setWaterReflectionTint(0.31,0.769,0.894);
-setWaterWaviness(3);
+setWaterWaviness(2.5);
+setWaterType("ocean");
 setWaterMurkiness(0.8);
-setWaterReflectionTintStrength(0.1);
 
 // Export map data
 ExportMap();
