@@ -74,10 +74,8 @@ function randomizeBiome()
 		// this assumes ocean settings, maps that aren't oceans should reset.
 		setWaterColour(0.114, 0.192, 0.463);
 		setWaterTint(0.255, 0.361, 0.651);
-		setWaterReflectionTint(0.255, 0.361, 0.651);
 		setWaterWaviness(5.5);
 		setWaterMurkiness(0.83);
-		setWaterReflectionTintStrength(0.05);
 		
 		setFogThickness(0.25);
 		setFogFactor(0.4);
@@ -182,10 +180,8 @@ function randomizeBiome()
 		// this assumes ocean settings, maps that aren't oceans should reset.
 		setWaterColour(0.067, 0.212, 0.361);
 		setWaterTint(0.4, 0.486, 0.765);
-		setWaterReflectionTint(0.482, 0.824, 0.925);
 		setWaterWaviness(5.5);
 		setWaterMurkiness(0.83);
-		setWaterReflectionTintStrength(0.15);
 
 		rbt1 = ["polar_snow_b", "snow grass 75", "snow rocks", "snow forest"];
 		rbt2 = "polar_tundra_snow";
@@ -242,10 +238,8 @@ function randomizeBiome()
 		// Went for a very clear, slightly blue-ish water in this case, basically no waves.
 		setWaterColour(0, 0.227, 0.843);
 		setWaterTint(0, 0.545, 0.859);
-		setWaterReflectionTint(1, 1, 1);
 		setWaterWaviness(1);
 		setWaterMurkiness(0.22);
-		setWaterReflectionTintStrength(0.0);
 		
 		setFogFactor(0.5);
 		setFogThickness(0.0);
@@ -321,10 +315,8 @@ function randomizeBiome()
 		// this is not intended for a clear running river, or even an ocean.
 		setWaterColour(0.0, 0.047, 0.286);				// dark majestic blue
 		setWaterTint(0.471, 0.776, 0.863);				// light blue
-		setWaterReflectionTint(0.0, 0.047, 0.286);
 		setWaterMurkiness(0.82);
 		setWaterWaviness(2);
-		setWaterReflectionTintStrength(0.15);
 
 		setFogThickness(0.26);
 		setFogFactor(0.4);
@@ -382,10 +374,8 @@ function randomizeBiome()
 		// Guess what, this is based on the colors of the mediterranean sea.
 		setWaterColour(0.024,0.212,0.024);
 		setWaterTint(0.133, 0.725,0.855);
-		setWaterReflectionTint(0.31,0.769,0.894);
 		setWaterWaviness(3);
 		setWaterMurkiness(0.8);
-		setWaterReflectionTintStrength(0.1);
 
 		setFogFactor(0.3);
 		setFogThickness(0.25);
@@ -479,10 +469,8 @@ function randomizeBiome()
 		// Using the Malawi as a reference, in parts where it's not too murky from a river nearby.
 		setWaterColour(0.055,0.176,0.431);
 		setWaterTint(0.227,0.749,0.549);
-		setWaterReflectionTint(1,1,1);
 		setWaterWaviness(1.5);
 		setWaterMurkiness(0.77);
-		setWaterReflectionTintStrength(0.0);
 
 		setFogFactor(0.25);
 		setFogThickness(0.15);
@@ -557,10 +545,8 @@ function randomizeBiome()
 		// Mostly for shallow maps. Maps where the water level goes deeper should use a much darker Water Colour to simulate deep water holes.
 		setWaterColour(0.584,0.824,0.929);
 		setWaterTint(0.569,0.965,0.945);
-		setWaterReflectionTint(1,1,1);
 		setWaterWaviness(1.5);
 		setWaterMurkiness(0.35);
-		setWaterReflectionTintStrength(0.0);
 
 		setFogFactor(0.4);
 		setFogThickness(0.2);
@@ -620,10 +606,8 @@ function randomizeBiome()
 		// this assumes ocean settings, maps that aren't oceans should reset.
 		setWaterColour(0.157, 0.149, 0.443);
 		setWaterTint(0.443,0.42,0.824);
-		setWaterReflectionTint(0.863,0.667,0.608);
 		setWaterWaviness(2.5);
 		setWaterMurkiness(0.83);
-		setWaterReflectionTintStrength(0.35);
 
 		setFogFactor(0.35);
 		setFogThickness(0.22);
