@@ -109,26 +109,47 @@ TileClass.prototype.countInRadius = function(cx, cy, radius, returnMembers)
 	var size = this.size;
 
 	var ymax = cy+radius;
+	var radius2 = radius*radius;
 	
 	for (var y = cy-radius; y <= ymax; y++)
 	{
 		var iy = Math.floor(y);
-		if(iy >= 0 && iy < size)
+		if (radius >= 27) // Switchover point before RangeOp actually performs better than a straight algorithm
 		{
-			var dy = y - cy;
-			var dx = Math.sqrt(radius*radius - dy*dy);
-			
-			var lowerX = Math.floor(cx - dx);
-			var upperX = Math.floor(cx + dx);
-			
-			var minX = (lowerX > 0 ? lowerX : 0);		
-			var maxX = (upperX < size ? upperX+1 : size);
-			
-			var total = maxX - minX;
-			var mem = this.rangeCount[iy].get(minX, maxX);
-			
-			members += mem;
-			nonMembers += total - mem;
+			if(iy >= 0 && iy < size)
+			{
+				var dy = y - cy;
+				var dx = Math.sqrt(radius*radius - dy*dy);
+				
+				var lowerX = Math.floor(cx - dx);
+				var upperX = Math.floor(cx + dx);
+				
+				var minX = (lowerX > 0 ? lowerX : 0);		
+				var maxX = (upperX < size ? upperX+1 : size);
+				
+				var total = maxX - minX;
+				var mem = this.rangeCount[iy].get(minX, maxX);
+				
+				members += mem;
+				nonMembers += total - mem;
+			}
+		}
+		else // Simply check the tiles one by one to find the number
+		{
+			var xmax = cx + radius;
+			for (var x = cx-radius; x <= xmax; x++)
+			{
+				var ix = Math.floor(x);
+				var dx = (ix - cx);
+				var dy = (iy - cy);
+				if (dx*dx + dy*dy <= radius2)
+					if (this.inclusionCount[ix] && this.inclusionCount[ix][iy] && this.inclusionCount[ix][iy] > 0)
+					{
+						members += 1;
+					}
+					else
+						nonMembers += 1;
+			}
 		}
 	}
 	
