@@ -606,10 +606,9 @@ setSkySet("sunny");
 setSunColour(0.711, 0.746, 0.574);	
 setWaterColour(0.541,0.506,0.416);
 setWaterTint(0.694,0.592,0.522);
-setWaterReflectionTint(0.447,0.369,0.31);
 setWaterMurkiness(1);
-setWaterWaviness(2.5);
-setWaterReflectionTintStrength(0.55);
+setWaterWaviness(3.0);
+setWaterType("lake");
 
 // Export map data
 ExportMap();
