@@ -19,13 +19,112 @@ function getCheatsData()
 	{
 		var currentCheat = parseJSONData("simulation/data/cheats/"+fileName+".json");
 		if (Object.keys(cheats).indexOf(currentCheat.Name) !== -1)
-			warn(sprintf("Cheat name '%(name)s' is already present", { name: currentCheat.Name }));
+			warn("Cheat name '" + currentCheat.Name + "' is already present");
 		else
 			cheats[currentCheat.Name] = currentCheat.Data;
 	}
 	return cheats;
 }
 
+var g_NotificationsTypes =
+{
+	"chat": function(notification, player)
+	{
+		var message = {
+			"type": "message",
+			"text": notification.message
+		}
+		var guid = findGuidForPlayerID(g_PlayerAssignments, player);
+		if (guid == undefined)
+		{
+			message["guid"] = -1;
+			message["player"] = player;
+		} else {
+			message["guid"] = guid;
+		}
+		addChatMessage(message);
+	},
+	"aichat": function(notification, player)
+	{
+		var message = {
+			"type": "message",
+			"text": notification.message
+		}
+		if (notification.type == "aichat")
+			message["translate"] = true;
+		var guid = findGuidForPlayerID(g_PlayerAssignments, player);
+		if (guid == undefined)
+		{
+			message["guid"] = -1;
+			message["player"] = player;
+		} else {
+			message["guid"] = guid;
+		}
+		addChatMessage(message);
+	},
+	"defeat": function(notification, player)
+	{
+		addChatMessage({
+			"type": "defeat",
+			"guid": findGuidForPlayerID(g_PlayerAssignments, player),
+			"player": player
+		});
+
+		// If the diplomacy panel is open refresh it.
+		if (isDiplomacyOpen)
+			openDiplomacy();
+	},
+	"diplomacy": function(notification, player)
+	{
+		addChatMessage({
+			"type": "diplomacy",
+			"player": player,
+			"player1": notification.player1,
+			"status": notification.status
+		});
+
+		// If the diplomacy panel is open refresh it.
+		if (isDiplomacyOpen)
+			openDiplomacy();
+	},
+	"quit": function(notification, player)
+	{
+		Engine.Exit();
+	},
+	"tribute": function(notification, player)
+	{
+		addChatMessage({
+			"type": "tribute",
+			"player": player,
+			"player1": notification.donator,
+			"amounts": notification.amounts
+		});
+	},
+	"attack": function(notification, player)
+	{
+		if (player != Engine.GetPlayerID())
+			return;
+		if (Engine.ConfigDB_GetValue("user", "gui.session.attacknotificationmessage") !== "true")
+			return;
+		addChatMessage({
+			"type": "attack",
+			"player": player,
+			"attacker": notification.attacker
+		});
+	},
+	"dialog": function(notification, player)
+	{
+		if (player == Engine.GetPlayerID())
+			openDialog(notification.dialogName, notification.data, player);
+	},
+	"resetselectionpannel": function(notification, player)
+	{
+		if (player != Engine.GetPlayerID())
+			return;
+		g_Selection.rebuildSelection([]);
+	}
+};
+
 // Notifications
 function handleNotifications()
 {
@@ -33,133 +132,41 @@ function handleNotifications()
 
 	if (!notification)
 		return;
-	
-	if (notification.type === undefined)
-		notification.type = "text";
+	if (!notification.type)
+	{
+		error("notification without type found.\n"+uneval(notification))
+		return;
+	}
+	if (!notification.players)
+	{
+		error("notification without players found.\n"+uneval(notification))
+		return;
+	}
+	var action = g_NotificationsTypes[notification.type];
+	if (!action)
+	{
+		error("unknown notification type '" + notification.type + "' found.");
+		return;
+	}
 
-	// Handle chat notifications specially
-	if (notification.type == "chat" ||
-		notification.type == "aichat")
-	{
-		var message = {
-			"type": "message",
-			"text": notification.message
-		}
-		if (notification.type == "aichat")
-			message["translate"] = true;
-		var guid = findGuidForPlayerID(g_PlayerAssignments, notification.player);
-		if (guid == undefined)
-		{
-			message["guid"] = -1;
-			message["player"] = notification.player;
-		} else {
-			message["guid"] = findGuidForPlayerID(g_PlayerAssignments, notification.player);
-		}
-		addChatMessage(message);
-	}
-	else if (notification.type == "defeat")
-	{
-		addChatMessage({
-			"type": "defeat",
-			"guid": findGuidForPlayerID(g_PlayerAssignments, notification.player),
-			"player": notification.player
-		});
-
-		// If the diplomacy panel is open refresh it.
-		if (isDiplomacyOpen)
-			openDiplomacy();
-	}
-	else if (notification.type == "diplomacy")
-	{
-		addChatMessage({
-			"type": "diplomacy",
-			"player": notification.player,
-			"player1": notification.player1,
-			"status": notification.status
-		});
-
-		// If the diplomacy panel is open refresh it.
-		if (isDiplomacyOpen)
-			openDiplomacy();
-	}
-	else if (notification.type == "quit")
-	{
-		// Used for AI testing
-		exit();
-	}
-	else if (notification.type == "tribute")
-	{
-		addChatMessage({
-			"type": "tribute",
-			"player": notification.player,
-			"player1": notification.player1,
-			"amounts": notification.amounts
-		});
-	}
-	else if (notification.type == "attack")
-	{
-		if (notification.player == Engine.GetPlayerID())
-		{
-			if (Engine.ConfigDB_GetValue("user", "gui.session.attacknotificationmessage") === "true")
-			{
-				addChatMessage({
-					"type": "attack",
-					"player": notification.player,
-					"attacker": notification.attacker
-				});
-			}
-		}
-	}
-	else if (notification.type == "text")
-	{
-		// Only display notifications directed to this player
-		if (notification.player == Engine.GetPlayerID())
-		{
-			notifications.push(notification);
-			notificationsTimers.push(setTimeout(removeOldNotifications, NOTIFICATION_TIMEOUT));
-
-			if (notifications.length > MAX_NUM_NOTIFICATION_LINES)
-				removeOldNotifications();
-			else
-				displayNotifications();
-		}
-	}
-	else
-	{
-		warn("notification of unknown type!");
-	}
-}
-
-function removeOldNotifications()
-{
-	clearTimeout(notificationsTimers[0]); // The timer only needs to be cleared when new notifications bump old notifications off
-	notificationsTimers.shift();
-	notifications.shift();
-	displayNotifications();
-}
-
-function displayNotifications()
-{
-	var messages = [];
-	for each (var n in notifications)
-	{
-		var parameters = n.parameters || {};
-		if (n.translateParameters)
-			translateObjectKeys(parameters, n.translateParameters);
-		var message = n.message;
-		if (n.translateMessage)
-			message = translate(message);
-		messages.push(sprintf(message, parameters));
-	}
-	Engine.GetGUIObjectByName("notificationText").caption = messages.join("\n");
+	for (var player of notification.players)
+		action(notification, player);
 }
 
 function updateTimeNotifications()
 {
 	var notifications =  Engine.GuiInterfaceCall("GetTimeNotifications");
 	var notificationText = "";
+	var playerID = Engine.GetPlayerID();
 	for (var n of notifications)
 	{
+		if (!n.players)
+		{
+			warn("notification has unknown player list. Text:\n"+n.message);
+			continue;
+		}
+		if (n.players.indexOf(playerID) == -1)
+			continue;
 		var message = n.message;
 		if (n.translateMessage)
 			message = translate(message);
@@ -169,7 +176,7 @@ function updateTimeNotifications()
 		parameters.time = timeToString(n.time);
 		notificationText += sprintf(message, parameters) + "\n";
 	}
-	Engine.GetGUIObjectByName("timeNotificationText").caption = notificationText;
+	Engine.GetGUIObjectByName("notificationText").caption = notificationText;
 }
 
 // Returns [username, playercolor] for the given player
@@ -224,7 +231,7 @@ function handleNetMessage(message)
 			obj.hidden = false;
 			break;
 		default:
-			error(sprintf("Unrecognised netstatus type %(netType)s", { netType: message.status }));
+			error("Unrecognised netstatus type '" + message.status + "'");
 			break;
 		}
 		break;
@@ -280,7 +287,7 @@ function handleNetMessage(message)
 		break;
 
 	default:
-		error(sprintf("Unrecognised net message type %(messageType)s", { messageType: message.type }));
+		error("Unrecognised net message type '" + message.type + "'");
 	}
 }
 
@@ -496,11 +503,6 @@ function addChatMessage(msg, playerAssignments)
 		{
 			if (msg.context !== "")
 			{
-				Engine.Console_Write(sprintf(translate("(%(context)s) * %(user)s %(message)s"), {
-					context: msg.context,
-					user: username,
-					message: message
-				}));
 				formatted = sprintf(translate("(%(context)s) * %(user)s %(message)s"), {
 					context: msg.context,
 					user: "[color=\"" + playerColor + "\"]" + username + "[/color]",
@@ -509,10 +511,6 @@ function addChatMessage(msg, playerAssignments)
 			}
 			else
 			{
-				Engine.Console_Write(sprintf(translate("* %(user)s %(message)s"), {
-					user: username,
-					message: message
-				}));
 				formatted = sprintf(translate("* %(user)s %(message)s"), {
 					user: "[color=\"" + playerColor + "\"]" + username + "[/color]",
 					message: message
@@ -525,11 +523,6 @@ function addChatMessage(msg, playerAssignments)
 			var formattedUserTag = sprintf(translate("<%(user)s>"), { user: "[color=\"" + playerColor + "\"]" + username + "[/color]" })
 			if (msg.context !== "")
 			{
-				Engine.Console_Write(sprintf(translate("(%(context)s) %(userTag)s %(message)s"), {
-					context: msg.context,
-					userTag: userTag,
-					message: message
-				}));
 				formatted = sprintf(translate("(%(context)s) %(userTag)s %(message)s"), {
 					context: msg.context,
 					userTag: formattedUserTag,
@@ -538,7 +531,6 @@ function addChatMessage(msg, playerAssignments)
 			}
 			else
 			{
-				Engine.Console_Write(sprintf(translate("%(userTag)s %(message)s"), { userTag: userTag, message: message}));
 				formatted = sprintf(translate("%(userTag)s %(message)s"), { userTag: formattedUserTag, message: message});
 			}
 		}
@@ -652,3 +644,55 @@ function parseChatCommands(msg, playerAssignments)
 	if (recurse)
 		parseChatCommands(msg, playerAssignments);
 }
+
+function sendDialogAnswer(guiObject, dialogName)
+{
+	Engine.GetGUIObjectByName(dialogName+"-dialog").hidden = true;
+
+	Engine.PostNetworkCommand({
+		"type": "dialog-answer",
+		"dialog": dialogName,
+		"answer": guiObject.name.split("-").pop(),
+	});
+
+	resumeGame();
+}
+
+function openDialog(dialogName, data, player)
+{
+	var dialog = Engine.GetGUIObjectByName(dialogName+"-dialog");
+	if (!dialog)
+	{
+		warn("messages.js: Unknow dialog with name "+dialogName);
+		return;
+	}
+	dialog.hidden = false;
+
+	for (var objName in data)
+	{
+		var obj = Engine.GetGUIObjectByName(dialogName + "-dialog-" + objName);
+		if (!obj)
+		{
+			warn("messages.js: Key '" + objName + "' not found in '" + dialogName + "' dialog.");
+			continue;
+		}
+		for (var key in data[objName])
+		{
+			var n = data[objName][key];
+			if (typeof n == "object" && n.message)
+			{
+				var message = n.message;
+				if (n.translateMessage)
+					message = translate(message);
+				var parameters = n.parameters || {};
+				if (n.translateParameters)
+					translateObjectKeys(parameters, n.translateParameters);
+				obj[key] = sprintf(message, parameters);
+			}
+			else
+				obj[key] = n;
+		}
+	}
+
+	pauseGame();
+}
