@@ -164,15 +164,15 @@ function displaySingle(entState, template)
 		Engine.GetGUIObjectByName("resourceCarryingIcon").tooltip = sprintf(translate("Gain: %(amount)s"), { amount: getTradingTooltip(entState.trader.goods.amount) });
 	}
 	// And for number of workers
-	else if (entState.foundation)
+	else if (entState.foundation && !entState.mirage)
 	{
 		Engine.GetGUIObjectByName("resourceCarryingIcon").hidden = false;
 		Engine.GetGUIObjectByName("resourceCarryingText").hidden = false;
 		Engine.GetGUIObjectByName("resourceCarryingIcon").sprite = "stretched:session/icons/repair.png";
 		Engine.GetGUIObjectByName("resourceCarryingText").caption = entState.foundation.numBuilders + "    ";
-		Engine.GetGUIObjectByName("resourceCarryingIcon").tooltip = translate("Number of builders");
+		Engine.GetGUIObjectByName("resourceCarryingIcon").tooltip = sprintf(translate("Number of builders.\nTasking another to this foundation would speed construction up by %(numb)s%%"), { numb : Math.round((Math.pow((entState.foundation.numBuilders+1)/entState.foundation.numBuilders, 0.7) - 1.0)*100) });
 	}
-	else if (entState.resourceSupply && (!entState.resourceSupply.killBeforeGather || !entState.hitpoints))
+	else if (entState.resourceSupply && (!entState.resourceSupply.killBeforeGather || !entState.hitpoints) && !entState.mirage)
 	{
 		Engine.GetGUIObjectByName("resourceCarryingIcon").hidden = false;
 		Engine.GetGUIObjectByName("resourceCarryingText").hidden = false;
@@ -302,6 +302,25 @@ function displaySingle(entState, template)
 	if (genericName)
 		iconTooltip = "[font=\"sans-bold-16\"]" + genericName + "[/font]";
 
+	if (template.visibleIdentityClasses && template.visibleIdentityClasses.length)
+	{
+		iconTooltip += "\n[font=\"sans-bold-13\"]" + translate("Classes:") + "[/font] ";
+		iconTooltip += "[font=\"sans-13\"]" + translate(template.visibleIdentityClasses[0]) ;
+		for (var i = 1; i < template.visibleIdentityClasses.length; i++)
+			iconTooltip += ", " + translate(template.visibleIdentityClasses[i]);
+		iconTooltip += "[/font]";
+	}
+
+	if (template.auras)
+	{
+		for (var auraName in template.auras)
+		{
+			iconTooltip += "\n[font=\"sans-bold-13\"]" + translate(auraName) + "[/font]";
+			if (template.auras[auraName])
+				iconTooltip += ": " + translate(template.auras[auraName]);
+		}
+	}
+
 	if (template.tooltip)
 		iconTooltip += "\n[font=\"sans-13\"]" + template.tooltip + "[/font]";
 
@@ -409,8 +428,5 @@ function updateSelectionDetails()
 	{
 		// Fill out commands panel for specific unit selected (or first unit of primary group)
 		updateUnitCommands(entState, supplementalDetailsPanel, commandsPanel, selection);
-		// Show panels
-		supplementalDetailsPanel.hidden = false;
-		commandsPanel.hidden = false;
 	}
 }
